import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level, min_sdk_level
import objc


class TestNSMenuHelper(AppKit.NSObject):
    def validateMenuItem_(self, item):
        return 1

    def numberOfItemsInMenu_(self, menu):
        return 1

    def menu_updateItem_atIndex_shouldCancel_(self, m, i, d, s):
        return 1

    def menuHasKeyEquivalent_forEvent_target_action_(self, m, e, t, a):
        return 1

    def confinementRectForMenu_onScreen_(self, m, s):
        return 1


class TestNSMenu(TestCase):
    @min_sdk_level("10.6")
    def testProtocolObjects(self):
        objc.protocolNamed("NSMenuDelegate")

    def testProtocol(self):
        self.assertResultIsBOOL(TestNSMenuHelper.validateMenuItem_)
        self.assertResultHasType(
            TestNSMenuHelper.numberOfItemsInMenu_, objc._C_NSInteger
        )
        self.assertResultIsBOOL(TestNSMenuHelper.menu_updateItem_atIndex_shouldCancel_)
        self.assertArgHasType(
            TestNSMenuHelper.menu_updateItem_atIndex_shouldCancel_, 2, objc._C_NSInteger
        )
        self.assertArgIsBOOL(TestNSMenuHelper.menu_updateItem_atIndex_shouldCancel_, 3)
        self.assertResultIsBOOL(
            TestNSMenuHelper.menuHasKeyEquivalent_forEvent_target_action_
        )
        self.assertArgHasType(
            TestNSMenuHelper.menuHasKeyEquivalent_forEvent_target_action_, 2, b"o^@"
        )
        self.assertArgHasType(
            TestNSMenuHelper.menuHasKeyEquivalent_forEvent_target_action_, 3, b"o^:"
        )

    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSMenu.menuBarVisible)
        self.assertArgIsBOOL(AppKit.NSMenu.setMenuBarVisible_, 0)
        self.assertResultIsBOOL(AppKit.NSMenu.autoenablesItems)
        self.assertArgIsBOOL(AppKit.NSMenu.setAutoenablesItems_, 0)
        self.assertResultIsBOOL(AppKit.NSMenu.performKeyEquivalent_)
        self.assertResultIsBOOL(AppKit.NSMenu.autoenablesItems)
        self.assertArgIsBOOL(AppKit.NSMenu.setMenuChangedMessagesEnabled_, 0)
        self.assertResultIsBOOL(AppKit.NSMenu.isTornOff)
        self.assertResultIsBOOL(AppKit.NSMenu.isAttached)
        self.assertResultIsBOOL(AppKit.NSMenu.showsStateColumn)
        self.assertArgIsBOOL(AppKit.NSMenu.setShowsStateColumn_, 0)

        self.assertResultIsBOOL(AppKit.NSMenu.menuChangedMessagesEnabled)
        self.assertArgIsBOOL(AppKit.NSMenu.setMenuChangedMessagesEnabled_, 0)
        self.assertResultHasType(
            AppKit.NSMenu.locationForSubmenu_, AppKit.NSPoint.__typestr__
        )

    def testConstants(self):
        self.assertIsInstance(AppKit.NSMenuWillSendActionNotification, str)
        self.assertIsInstance(AppKit.NSMenuDidSendActionNotification, str)
        self.assertIsInstance(AppKit.NSMenuDidAddItemNotification, str)
        self.assertIsInstance(AppKit.NSMenuDidRemoveItemNotification, str)
        self.assertIsInstance(AppKit.NSMenuDidChangeItemNotification, str)
        self.assertIsInstance(AppKit.NSMenuDidBeginTrackingNotification, str)
        self.assertIsInstance(AppKit.NSMenuDidEndTrackingNotification, str)

    @min_os_level("10.6")
    def testMethods10_6(self):
        self.assertResultIsBOOL(
            AppKit.NSMenu.popUpMenuPositioningItem_atLocation_inView_
        )
        self.assertArgHasType(
            AppKit.NSMenu.popUpMenuPositioningItem_atLocation_inView_,
            1,
            AppKit.NSPoint.__typestr__,
        )
        self.assertResultHasType(AppKit.NSMenu.size, AppKit.NSSize.__typestr__)
        self.assertResultIsBOOL(AppKit.NSMenu.allowsContextMenuPlugIns)
        self.assertArgIsBOOL(AppKit.NSMenu.setAllowsContextMenuPlugIns_, 0)

        self.assertResultHasType(
            TestNSMenuHelper.confinementRectForMenu_onScreen_, AppKit.NSRect.__typestr__
        )

    @min_os_level("10.6")
    def testConstants10_6(self):
        self.assertEqual(AppKit.NSMenuPropertyItemTitle, 1 << 0)
        self.assertEqual(AppKit.NSMenuPropertyItemAttributedTitle, 1 << 1)
        self.assertEqual(AppKit.NSMenuPropertyItemKeyEquivalent, 1 << 2)
        self.assertEqual(AppKit.NSMenuPropertyItemImage, 1 << 3)
        self.assertEqual(AppKit.NSMenuPropertyItemEnabled, 1 << 4)
        self.assertEqual(AppKit.NSMenuPropertyItemAccessibilityDescription, 1 << 5)



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level, min_sdk_level
import objc


class TestNSCandidateListTouchBarItemHelper(AppKit.NSObject):
    def candidateListTouchBarItem_beginSelectingCandidateAtIndex_(self, item, index):
        pass

    def candidateListTouchBarItem_changeSelectionFromCandidateAtIndex_toIndex_(
        self, item, index, index2
    ):
        pass

    def candidateListTouchBarItem_endSelectingCandidateAtIndex_(self, item, index):
        pass

    def candidateListTouchBarItem_changedCandidateListVisibility_(self, item, flag):
        pass


class TestNSCandidateListTouchBarItem(TestCase):
    @min_sdk_level("10.12")
    def testProtocoObjects10_12(self):
        objc.protocolNamed("NSCandidateListTouchBarItemDelegate")

    @min_os_level("10.12")
    def testConstants10_12(self):
        self.assertIsInstance(AppKit.NSTouchBarItemIdentifierCandidateList, str)

    @min_os_level("10.12")
    def testMethods10_12(self):
        self.assertResultIsBOOL(AppKit.NSCandidateListTouchBarItem.isCollapsed)
        self.assertArgIsBOOL(AppKit.NSCandidateListTouchBarItem.setCollapsed_, 0)

        self.assertResultIsBOOL(AppKit.NSCandidateListTouchBarItem.allowsCollapsing)
        self.assertArgIsBOOL(AppKit.NSCandidateListTouchBarItem.setAllowsCollapsing_, 0)

        self.assertResultIsBOOL(
            AppKit.NSCandidateListTouchBarItem.isCandidateListVisible
        )
        # self.assertArgIsBOOL(AppKit.NSCandidateListTouchBarItem.setCandidateListVisible_, 0)

        self.assertArgIsBOOL(
            AppKit.NSCandidateListTouchBarItem.updateWithInsertionPointVisibility_, 0
        )

        self.assertResultIsBOOL(
            AppKit.NSCandidateListTouchBarItem.allowsTextInputContextCandidates
        )
        self.assertArgIsBOOL(
            AppKit.NSCandidateListTouchBarItem.setAllowsTextInputContextCandidates_, 0
        )

        self.assertResultIsBlock(
            AppKit.NSCandidateListTouchBarItem.attributedStringForCandidate,
            b"@@" + objc._C_NSInteger,
        )
        self.assertArgIsBlock(
            AppKit.NSCandidateListTouchBarItem.setAttributedStringForCandidate_,
            0,
            b"@@" + objc._C_NSInteger,
        )

        self.assertArgHasType(
            TestNSCandidateListTouchBarItemHelper.candidateListTouchBarItem_beginSelectingCandidateAtIndex_,  # noqa: B950
            1,
            objc._C_NSInteger,
        )
        self.assertArgHasType(
            TestNSCandidateListTouchBarItemHelper.candidateListTouchBarItem_changeSelectionFromCandidateAtIndex_toIndex_,  # noqa: B950
            1,
            objc._C_NSInteger,
        )
        self.assertArgHasType(
            TestNSCandidateListTouchBarItemHelper.candidateListTouchBarItem_changeSelectionFromCandidateAtIndex_toIndex_,  # noqa: B950
            2,
            objc._C_NSInteger,
        )
        self.assertArgHasType(
            TestNSCandidateListTouchBarItemHelper.candidateListTouchBarItem_endSelectingCandidateAtIndex_,  # noqa: B950
            1,
            objc._C_NSInteger,
        )
        self.assertArgIsBOOL(
            TestNSCandidateListTouchBarItemHelper.candidateListTouchBarItem_changedCandidateListVisibility_,  # noqa: B950
            1,
        )



import AppKit
from PyObjCTools.TestSupport import TestCase


class TestNSStepper(TestCase):
    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSStepper.valueWraps)
        self.assertArgIsBOOL(AppKit.NSStepper.setValueWraps_, 0)
        self.assertResultIsBOOL(AppKit.NSStepper.autorepeat)
        self.assertArgIsBOOL(AppKit.NSStepper.setAutorepeat_, 0)



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSCell(TestCase):
    def testUnicode(self):
        u = "\xc3\xbc\xc3\xb1\xc3\xae\xc3\xa7\xc3\xb8d\xc3\xa8"
        cell = AppKit.NSCell.alloc().initTextCell_(u)
        cell.setStringValue_(u)
        self.assertEqual(cell.stringValue(), u)

    def testInt(self):
        i = 17
        cell = AppKit.NSCell.alloc().initTextCell_("")
        cell.setIntValue_(i)
        self.assertEqual(cell.intValue(), i)

    def testFloat(self):
        f = 3.125
        cell = AppKit.NSCell.alloc().initTextCell_("")
        cell.setFloatValue_(f)
        self.assertEqual(cell.floatValue(), f)

    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSCell.prefersTrackingUntilMouseUp)
        self.assertResultIsBOOL(AppKit.NSCell.isOpaque)
        self.assertResultIsBOOL(AppKit.NSCell.isEnabled)
        self.assertArgIsBOOL(AppKit.NSCell.setEnabled_, 0)
        self.assertResultIsBOOL(AppKit.NSCell.isContinuous)
        self.assertArgIsBOOL(AppKit.NSCell.setContinuous_, 0)
        self.assertResultIsBOOL(AppKit.NSCell.isEditable)
        self.assertArgIsBOOL(AppKit.NSCell.setEditable_, 0)
        self.assertResultIsBOOL(AppKit.NSCell.isSelectable)
        self.assertArgIsBOOL(AppKit.NSCell.setSelectable_, 0)
        self.assertResultIsBOOL(AppKit.NSCell.isBordered)
        self.assertArgIsBOOL(AppKit.NSCell.setBordered_, 0)
        self.assertResultIsBOOL(AppKit.NSCell.isBezeled)
        self.assertArgIsBOOL(AppKit.NSCell.setBezeled_, 0)
        self.assertResultIsBOOL(AppKit.NSCell.isScrollable)
        self.assertArgIsBOOL(AppKit.NSCell.setScrollable_, 0)
        self.assertResultIsBOOL(AppKit.NSCell.isHighlighted)
        self.assertArgIsBOOL(AppKit.NSCell.setHighlighted_, 0)
        self.assertResultIsBOOL(AppKit.NSCell.wraps)
        self.assertArgIsBOOL(AppKit.NSCell.setWraps_, 0)
        self.assertResultIsBOOL(AppKit.NSCell.isEntryAcceptable_)
        self.assertArgIsBOOL(AppKit.NSCell.setFloatingPointFormat_left_right_, 0)
        self.assertResultIsBOOL(AppKit.NSCell.hasValidObjectValue)
        self.assertArgIsBOOL(AppKit.NSCell.highlight_withFrame_inView_, 0)
        self.assertArgIsOut(AppKit.NSCell.getPeriodicDelay_interval_, 0)
        self.assertArgIsOut(AppKit.NSCell.getPeriodicDelay_interval_, 1)
        self.assertResultIsBOOL(AppKit.NSCell.startTrackingAt_inView_)
        self.assertResultIsBOOL(AppKit.NSCell.continueTracking_at_inView_)
        self.assertArgIsBOOL(AppKit.NSCell.stopTracking_at_inView_mouseIsUp_, 3)
        self.assertResultIsBOOL(AppKit.NSCell.trackMouse_inRect_ofView_untilMouseUp_)
        self.assertArgIsBOOL(AppKit.NSCell.trackMouse_inRect_ofView_untilMouseUp_, 3)
        self.assertResultIsBOOL(AppKit.NSCell.sendsActionOnEndEditing)
        self.assertArgIsBOOL(AppKit.NSCell.setSendsActionOnEndEditing_, 0)
        self.assertResultIsBOOL(AppKit.NSCell.allowsUndo)
        self.assertArgIsBOOL(AppKit.NSCell.setAllowsUndo_, 0)
        self.assertResultIsBOOL(AppKit.NSCell.truncatesLastVisibleLine)
        self.assertArgIsBOOL(AppKit.NSCell.setTruncatesLastVisibleLine_, 0)
        self.assertResultIsBOOL(AppKit.NSCell.refusesFirstResponder)
        self.assertArgIsBOOL(AppKit.NSCell.setRefusesFirstResponder_, 0)
        self.assertResultIsBOOL(AppKit.NSCell.acceptsFirstResponder)
        self.assertResultIsBOOL(AppKit.NSCell.showsFirstResponder)
        self.assertArgIsBOOL(AppKit.NSCell.setShowsFirstResponder_, 0)
        self.assertResultIsBOOL(AppKit.NSCell.wantsNotificationForMarkedText)
        self.assertResultIsBOOL(AppKit.NSCell.allowsEditingTextAttributes)
        self.assertArgIsBOOL(AppKit.NSCell.setAllowsEditingTextAttributes_, 0)
        self.assertResultIsBOOL(AppKit.NSCell.importsGraphics)
        self.assertArgIsBOOL(AppKit.NSCell.setImportsGraphics_, 0)
        self.assertResultIsBOOL(AppKit.NSCell.allowsMixedState)
        self.assertArgIsBOOL(AppKit.NSCell.setAllowsMixedState_, 0)

    def testFunctions(self):
        self.assertArgIsBOOL(AppKit.NSDrawThreePartImage, 4)
        self.assertArgIsBOOL(AppKit.NSDrawThreePartImage, 7)
        self.assertArgIsBOOL(AppKit.NSDrawNinePartImage, 12)

    def testConstants(self):
        self.assertIsInstance(AppKit.NSControlTintDidChangeNotification, str)

        self.assertEqual(AppKit.NSAnyType, 0)
        self.assertEqual(AppKit.NSIntType, 1)
        self.assertEqual(AppKit.NSPositiveIntType, 2)
        self.assertEqual(AppKit.NSFloatType, 3)
        self.assertEqual(AppKit.NSPositiveFloatType, 4)
        self.assertEqual(AppKit.NSDoubleType, 6)
        self.assertEqual(AppKit.NSPositiveDoubleType, 7)

        self.assertEqual(AppKit.NSNullCellType, 0)
        self.assertEqual(AppKit.NSTextCellType, 1)
        self.assertEqual(AppKit.NSImageCellType, 2)

        self.assertEqual(AppKit.NSCellDisabled, 0)
        self.assertEqual(AppKit.NSCellState, 1)
        self.assertEqual(AppKit.NSPushInCell, 2)
        self.assertEqual(AppKit.NSCellEditable, 3)
        self.assertEqual(AppKit.NSChangeGrayCell, 4)
        self.assertEqual(AppKit.NSCellHighlighted, 5)
        self.assertEqual(AppKit.NSCellLightsByContents, 6)
        self.assertEqual(AppKit.NSCellLightsByGray, 7)
        self.assertEqual(AppKit.NSChangeBackgroundCell, 8)
        self.assertEqual(AppKit.NSCellLightsByBackground, 9)
        self.assertEqual(AppKit.NSCellIsBordered, 10)
        self.assertEqual(AppKit.NSCellHasOverlappingImage, 11)
        self.assertEqual(AppKit.NSCellHasImageHorizontal, 12)
        self.assertEqual(AppKit.NSCellHasImageOnLeftOrBottom, 13)
        self.assertEqual(AppKit.NSCellChangesContents, 14)
        self.assertEqual(AppKit.NSCellIsInsetButton, 15)
        self.assertEqual(AppKit.NSCellAllowsMixedState, 16)

        self.assertEqual(AppKit.NSNoImage, 0)
        self.assertEqual(AppKit.NSImageOnly, 1)
        self.assertEqual(AppKit.NSImageLeft, 2)
        self.assertEqual(AppKit.NSImageRight, 3)
        self.assertEqual(AppKit.NSImageBelow, 4)
        self.assertEqual(AppKit.NSImageAbove, 5)
        self.assertEqual(AppKit.NSImageOverlaps, 6)
        self.assertEqual(AppKit.NSImageLeading, 7)
        self.assertEqual(AppKit.NSImageTrailing, 8)

        self.assertEqual(AppKit.NSScaleProportionally, 0)
        self.assertEqual(AppKit.NSScaleToFit, 1)
        self.assertEqual(AppKit.NSScaleNone, 2)

        self.assertEqual(AppKit.NSImageScaleProportionallyDown, 0)
        self.assertEqual(AppKit.NSImageScaleAxesIndependently, 1)
        self.assertEqual(AppKit.NSImageScaleNone, 2)
        self.assertEqual(AppKit.NSImageScaleProportionallyUpOrDown, 3)

        self.assertEqual(AppKit.NSMixedState, -1)
        self.assertEqual(AppKit.NSOffState, 0)
        self.assertEqual(AppKit.NSOnState, 1)

        self.assertEqual(AppKit.NSControlStateMixed, -1)
        self.assertEqual(AppKit.NSControlStateOff, 0)
        self.assertEqual(AppKit.NSControlStateOn, 1)

        self.assertEqual(AppKit.NSNoCellMask, 0)
        self.assertEqual(AppKit.NSContentsCellMask, 1)
        self.assertEqual(AppKit.NSPushInCellMask, 2)
        self.assertEqual(AppKit.NSChangeGrayCellMask, 4)
        self.assertEqual(AppKit.NSChangeBackgroundCellMask, 8)

        self.assertEqual(AppKit.NSDefaultControlTint, 0)
        self.assertEqual(AppKit.NSBlueControlTint, 1)
        self.assertEqual(AppKit.NSGraphiteControlTint, 6)
        self.assertEqual(AppKit.NSClearControlTint, 7)

        self.assertEqual(AppKit.NSRegularControlSize, 0)
        self.assertEqual(AppKit.NSSmallControlSize, 1)
        self.assertEqual(AppKit.NSMiniControlSize, 2)

        self.assertEqual(AppKit.NSControlSizeRegular, 0)
        self.assertEqual(AppKit.NSControlSizeSmall, 1)
        self.assertEqual(AppKit.NSControlSizeMini, 2)

        self.assertEqual(AppKit.NSCellHitNone, 0)
        self.assertEqual(AppKit.NSCellHitContentArea, 1 << 0)
        self.assertEqual(AppKit.NSCellHitEditableTextArea, 1 << 1)
        self.assertEqual(AppKit.NSCellHitTrackableArea, 1 << 2)

        self.assertEqual(AppKit.NSBackgroundStyleLight, 0)
        self.assertEqual(AppKit.NSBackgroundStyleDark, 1)
        self.assertEqual(AppKit.NSBackgroundStyleRaised, 2)
        self.assertEqual(AppKit.NSBackgroundStyleLowered, 3)

        self.assertEqual(AppKit.NSControlStateValueMixed, -1)
        self.assertEqual(AppKit.NSControlStateValueOff, 0)
        self.assertEqual(AppKit.NSControlStateValueOn, 1)

        self.assertEqual(AppKit.NSBackgroundStyleNormal, 0)
        self.assertEqual(AppKit.NSBackgroundStyleEmphasized, 1)
        self.assertEqual(AppKit.NSBackgroundStyleRaised, 2)
        self.assertEqual(AppKit.NSBackgroundStyleLowered, 3)

    @min_os_level("10.6")
    def testMethods10_6(self):
        self.assertResultIsBOOL(AppKit.NSCell.usesSingleLineMode)
        self.assertArgIsBOOL(AppKit.NSCell.setUsesSingleLineMode_, 0)



import Foundation
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSUbiquitousKeyValueStore(TestCase):
    @min_os_level("10.7")
    def testMethods10_7(self):
        self.assertResultIsBOOL(Foundation.NSUbiquitousKeyValueStore.boolForKey_)
        self.assertArgIsBOOL(Foundation.NSUbiquitousKeyValueStore.setBool_forKey_, 0)
        self.assertResultIsBOOL(Foundation.NSUbiquitousKeyValueStore.synchronize)

    @min_os_level("10.7")
    def testConstants10_7(self):
        self.assertIsInstance(
            Foundation.NSUbiquitousKeyValueStoreDidChangeExternallyNotification, str
        )
        self.assertIsInstance(Foundation.NSUbiquitousKeyValueStoreChangeReasonKey, str)
        self.assertIsInstance(Foundation.NSUbiquitousKeyValueStoreChangedKeysKey, str)

        self.assertEqual(Foundation.NSUbiquitousKeyValueStoreServerChange, 0)
        self.assertEqual(Foundation.NSUbiquitousKeyValueStoreInitialSyncChange, 1)
        self.assertEqual(Foundation.NSUbiquitousKeyValueStoreQuotaViolationChange, 2)

    @min_os_level("10.8")
    def testConstants10_8(self):
        self.assertEqual(Foundation.NSUbiquitousKeyValueStoreAccountChange, 3)



# Updated by mvl on 2009-02-22 for PyObjC 2

import Cocoa
import objc
from objc import super
from PyObjCTools import AppHelper


class DraggableItemView(Cocoa.NSView):
    """."""

    _locationDefault = Cocoa.NSMakePoint(0.0, 0.0)
    _itemColorDefault = Cocoa.NSColor.redColor()
    _backgroundColorDefault = Cocoa.NSColor.whiteColor()

    def awakeFromNib(self):
        self.dragging = None

    def initWithFrame_(self, frame):
        """."""
        result = super().initWithFrame_(frame)
        if result is not None:
            result._location = self._locationDefault
            result._itemColor = self._itemColorDefault
            result._backgroundColor = self._backgroundColorDefault
        return result

    def drawRect_(self, rect):
        """."""
        Cocoa.NSColor.whiteColor().set()
        Cocoa.NSBezierPath.fillRect_(rect)
        self.itemColor().set()
        Cocoa.NSBezierPath.fillRect_(self.calculatedItemBounds())

    def isOpaque(self):
        """."""
        return self.backgroundColor().alphaComponent() >= 1.0

    def offsetLocationByX_andY_(self, x, y):
        """."""
        self.setNeedsDisplayInRect_(self.calculatedItemBounds())
        if self.isFlipped():
            invertDeltaY = -1
        else:
            invertDeltaY = 1
        self.location().x = self.location().x + x
        self.location().y = self.location().y + y * invertDeltaY
        self.setNeedsDisplayInRect_(self.calculatedItemBounds())

    def mouseDown_(self, event):
        """."""
        clickLocation = self.convertPoint_fromView_(event.locationInWindow(), None)
        itemHit = self.isPointInItem_(clickLocation)
        if itemHit:
            self.dragging = True
            self.lastDragLocation = clickLocation
            Cocoa.NSCursor.closedHandCursor().push()

    def mouseDragged_(self, event):
        """."""
        if self.dragging:
            newDragLocation = self.convertPoint_fromView_(
                event.locationInWindow(), None
            )
            self.offsetLocationByX_andY_(
                newDragLocation.x - self.lastDragLocation.x,
                newDragLocation.y - self.lastDragLocation.y,
            )
            self.lastDragLocation = newDragLocation
            self.autoscroll_(event)

    def mouseUp_(self, event):
        """."""
        self.dragging = False
        Cocoa.NSCursor.pop()
        self.window().invalidateCursorRectsForView_(self)

    def acceptsFirstResponder(self):
        """."""
        return True

    def keyDown_(self, event):
        """."""
        handled = False
        characters = event.charactersIgnoringModifiers()
        if characters.isEqual_("r"):
            handled = True
            self.setItemPropertiesToDefault_(self)
        if handled is False:
            super().keyDown_(event)

    @objc.IBAction
    def changeColor_(self, sender):
        """."""
        self.setItemColor_(sender.color())

    def resetCursorRects(self):
        """."""
        self.discardCursorRects()
        self.addCursorRect_cursor_(
            self.calculatedItemBounds(), Cocoa.NSCursor.openHandCursor()
        )

    @objc.IBAction
    def moveUp_(self, sender):
        """."""
        self.offsetLocationByX_andY_(0.0, 10.0)
        self.window().invalidateCursorRectsForView_(self)

    @objc.IBAction
    def moveDown_(self, sender):
        """."""
        self.offsetLocationByX_andY_(0.0, -10.0)
        self.window().invalidateCursorRectsForView_(self)

    @objc.IBAction
    def moveLeft_(self, sender):
        """."""
        self.offsetLocationByX_andY_(-10.0, 0.0)
        self.window().invalidateCursorRectsForView_(self)

    @objc.IBAction
    def moveRight_(self, sender):
        """."""
        self.offsetLocationByX_andY_(10.0, 0.0)
        self.window().invalidateCursorRectsForView_(self)

    @objc.IBAction
    def setItemPropertiesToDefault_(self, sender):
        """."""
        self.setLocation_(self._locationDefault)
        self.setItemColor_(self._itemColorDefault)
        self.setBackgroundColor_(self._backgroundColorDefault)

    def setLocation_(self, point):
        """."""
        if not Cocoa.NSEqualPoints(point, self.location()):
            self.setNeedsDisplayInRect_(self.calculatedItemBounds())
            self._location = point
            self.setNeedsDisplayInRect_(self.calculatedItemBounds())
            self.window().invalidateCursorRectsForView_(self)

    def location(self):
        """."""
        return self._location

    def setBackgroundColor_(self, aColor):
        """."""
        if not self.backgroundColor().isEqual_(aColor):
            self._backgroundColor = aColor
            self.setNeedsDisplayInRect_(self.calculatedItemBounds())

    def backgroundColor(self):
        """."""
        return self._backgroundColor

    def setItemColor_(self, aColor):
        """."""
        if not self.itemColor().isEqual_(aColor):
            self._itemColor = aColor
            self.setNeedsDisplayInRect_(self.calculatedItemBounds())

    def itemColor(self):
        """."""
        return self._itemColor

    def calculatedItemBounds(self):
        """."""
        return Cocoa.NSMakeRect(self.location().x, self.location().y, 60.0, 20.0)

    def isPointInItem_(self, testPoint):
        """."""
        itemHit = Cocoa.NSPointInRect(testPoint, self.calculatedItemBounds())
        if itemHit:
            pass
        return itemHit


if __name__ == "__main__":
    AppHelper.runEventLoop()



import Foundation
from PyObjCTools.TestSupport import TestCase, min_os_level
import objc


class TestNSObjCRuntime(TestCase):
    def testConstants(self):
        self.assertEqual(Foundation.NSQualityOfServiceUserInteractive, 0x21)
        self.assertEqual(Foundation.NSQualityOfServiceUserInitiated, 0x19)
        self.assertEqual(Foundation.NSQualityOfServiceUtility, 0x11)
        self.assertEqual(Foundation.NSQualityOfServiceBackground, 0x09)
        self.assertEqual(Foundation.NSQualityOfServiceDefault, -1)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_0, 397.40)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_1, 425.00)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_1_1, 425.00)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_1_2, 425.00)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_1_3, 425.00)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_1_4, 425.00)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_2, 462.00)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_2_1, 462.00)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_2_2, 462.00)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_2_3, 462.00)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_2_4, 462.00)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_2_5, 462.00)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_2_6, 462.00)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_2_7, 462.70)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_2_8, 462.70)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_3, 500.00)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_3_1, 500.00)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_3_2, 500.30)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_3_3, 500.54)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_3_4, 500.56)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_3_5, 500.56)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_3_6, 500.56)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_3_7, 500.56)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_3_8, 500.56)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_3_9, 500.58)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_4, 567.00)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_4_1, 567.00)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_4_2, 567.12)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_4_3, 567.21)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_4_4_Intel, 567.23)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_4_4_PowerPC, 567.21)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_4_5, 567.25)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_4_6, 567.26)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_4_7, 567.27)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_4_8, 567.28)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_4_9, 567.29)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_4_10, 567.29)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_4_11, 567.36)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_5, 677.00)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_5_1, 677.10)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_5_2, 677.15)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_5_3, 677.19)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_5_4, 677.19)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_5_5, 677.21)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_5_6, 677.22)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_5_7, 677.24)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_5_8, 677.26)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_6, 751.00)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_6_1, 751.00)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_6_2, 751.14)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_6_3, 751.21)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_6_4, 751.29)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_6_5, 751.42)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_6_6, 751.53)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_6_7, 751.53)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_6_8, 751.62)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_8, 945.00)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_8_1, 945.00)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_8_2, 945.11)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_8_3, 945.16)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_8_4, 945.18)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_9, 1056)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_9_1, 1056)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_9_2, 1056.13)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_10, 1151.16)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_10_1, 1151.16)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_10_2, 1152.14)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_10_3, 1153.20)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_10_4, 1153.20)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_10_5, 1154)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_10_Max, 1199)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_11, 1252)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_11_1, 1255.1)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_11_2, 1256.1)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_11_3, 1256.1)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_11_4, 1258)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_11_Max, 1299)

        self.assertIsInstance(Foundation.NSFoundationVersionNumber, float)
        self.assertIsInstance(Foundation.NSIntegerMax, int)
        self.assertIsInstance(Foundation.NSIntegerMin, int)
        self.assertIsInstance(Foundation.NSUIntegerMax, int)
        self.assertEqual(Foundation.NSIntegerMax, 2 ** 63 - 1)
        self.assertEqual(Foundation.NSIntegerMin, -(2 ** 63))
        self.assertEqual(Foundation.NSUIntegerMax, 2 ** 64 - 1)

        self.assertTrue(objc.YES)
        self.assertFalse(objc.NO)

        self.assertEqual(Foundation.NSOrderedAscending, -1)
        self.assertEqual(Foundation.NSOrderedSame, 0)
        self.assertEqual(Foundation.NSOrderedDescending, 1)

        self.assertEqual(Foundation.NSNotFound, Foundation.NSIntegerMax)

    @min_os_level("10.6")
    def testConstants10_6(self):
        self.assertEqual(Foundation.NSFoundationVersionNumber10_5, 677.00)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_5_1, 677.10)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_5_2, 677.15)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_5_3, 677.19)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_5_4, 677.19)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_5_5, 677.21)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_5_6, 677.22)

        self.assertEqual(Foundation.NSEnumerationConcurrent, 1 << 0)
        self.assertEqual(Foundation.NSEnumerationReverse, 1 << 1)
        self.assertEqual(Foundation.NSSortConcurrent, 1 << 0)
        self.assertEqual(Foundation.NSSortStable, 1 << 4)

    @min_os_level("10.7")
    def testConstants10_7(self):
        self.assertEqual(Foundation.NSFoundationVersionNumber10_7, 833.10)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_7_1, 833.10)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_7_2, 833.20)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_7_3, 833.24)
        self.assertEqual(Foundation.NSFoundationVersionNumber10_7_4, 833.25)

    def testSelectorAccess(self):
        v = Foundation.NSStringFromSelector("description")
        self.assertIsInstance(v, str)
        self.assertEqual(v, "description")

        v = Foundation.NSSelectorFromString("description")
        self.assertIsInstance(v, str)
        self.assertEqual(v, "description")

    def testClassAccess(self):
        v = Foundation.NSStringFromClass(Foundation.NSObject)
        self.assertIsInstance(v, str)
        self.assertEqual(v, "NSObject")

        v = Foundation.NSClassFromString("NSDictionary")
        self.assertIsInstance(v, objc.objc_class)
        self.assertEqual(v, Foundation.NSDictionary)

    def testProtocolAccess(self):
        p = Foundation.NSProtocolFromString("NSObject")
        self.assertIsInstance(p, objc.formal_protocol)
        v = Foundation.NSStringFromProtocol(p)
        self.assertIsInstance(v, str)
        self.assertEqual(v, "NSObject")

    def testTypeInfo(self):
        rest, size, align = Foundation.NSGetSizeAndAlignment(b"ii", None, None)
        self.assertEqual(rest, b"i")
        self.assertIsInstance(size, int)
        self.assertIsInstance(align, int)

    def testMinMax(self):
        self.assertEqual(Foundation.MAX(1, 2), 2)
        self.assertEqual(Foundation.MAX(2, 1), 2)
        self.assertEqual(Foundation.MAX("a", "b"), "b")
        self.assertEqual(Foundation.MIN(1, 2), 1)
        self.assertEqual(Foundation.MIN(2, 1), 1)
        self.assertEqual(Foundation.MIN("a", "b"), "a")
        self.assertEqual(Foundation.ABS(1), 1)
        self.assertEqual(Foundation.ABS(-1), 1)
        self.assertEqual(Foundation.ABS(-1.0), 1.0)

    def testFunctions(self):
        self.assertArgIsPrintf(Foundation.NSLog, 0)



import AppKit
from PyObjCTools.TestSupport import TestCase


class TestNSItemProviderReadingWritingHelper(AppKit.NSObject):
    def loadDataWithTypeIdentifier_forItemProviderCompletionHandler_(self, ti, ch):
        pass

    def initWithItemProviderData_typeIdentifier_error_(self, d, ti, e):
        pass

    @classmethod
    def objectWithItemProviderData_typeIdentifier_error_(self, d, ti, e):
        pass


class TestNSItemProviderReadingWriting(TestCase):
    def testMethods(self):
        self.assertArgIsBlock(
            TestNSItemProviderReadingWritingHelper.loadDataWithTypeIdentifier_forItemProviderCompletionHandler_,  # noqa: B950
            1,
            b"v@@",
        )
        self.assertArgIsOut(
            TestNSItemProviderReadingWritingHelper.initWithItemProviderData_typeIdentifier_error_,  # noqa: B950
            2,
        )

        self.assertArgIsOut(
            TestNSItemProviderReadingWritingHelper.objectWithItemProviderData_typeIdentifier_error_,  # noqa: B950
            2,
        )



import Foundation
from PyObjCTools.TestSupport import TestCase


class TestArchiver(TestCase):
    def testConstants(self):
        self.assertIsInstance(Foundation.NSInconsistentArchiveException, str)

    def testMethods(self):
        self.assertResultIsBOOL(Foundation.NSArchiver.archiveRootObject_toFile_)
        self.assertResultIsBOOL(Foundation.NSUnarchiver.isAtEnd)



import objc
from Foundation import NSObject
from objc import super


class RPCMethod(NSObject):
    def initWithDocument_name_(self, aDocument, aName):
        self = super().init()
        self.document = aDocument
        self.k_methodName = aName
        self.k_methodSignature = None
        self.k_methodDescription = None
        return self

    def methodName(self):
        return self.k_methodName

    def displayName(self):
        if self.k_methodSignature is None:
            return self.k_methodName
        else:
            return self.k_methodSignature

    @objc.accessor
    def setMethodSignature_(self, aSignature):
        self.k_methodSignature = aSignature

    def methodDescription(self):
        if self.k_methodDescription is None:
            self.setMethodDescription_("<description not yet received>")
            self.document.fetchMethodDescription_(self)
        return self.k_methodDescription

    @objc.accessor
    def setMethodDescription_(self, aDescription):
        self.k_methodDescription = aDescription



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSTrackingArea(TestCase):
    @min_os_level("10.5")
    def testConstants(self):
        self.assertEqual(AppKit.NSTrackingMouseEnteredAndExited, 0x01)
        self.assertEqual(AppKit.NSTrackingMouseMoved, 0x02)
        self.assertEqual(AppKit.NSTrackingCursorUpdate, 0x04)

        self.assertEqual(AppKit.NSTrackingActiveWhenFirstResponder, 0x10)
        self.assertEqual(AppKit.NSTrackingActiveInKeyWindow, 0x20)
        self.assertEqual(AppKit.NSTrackingActiveInActiveApp, 0x40)
        self.assertEqual(AppKit.NSTrackingActiveAlways, 0x80)

        self.assertEqual(AppKit.NSTrackingAssumeInside, 0x100)
        self.assertEqual(AppKit.NSTrackingInVisibleRect, 0x200)
        self.assertEqual(AppKit.NSTrackingEnabledDuringMouseDrag, 0x400)



#
#  ControlledPreferences
#
#

# import classes required to start application
import FontNameToDisplayNameTransformer  # noqa: F401
import FontSampleDisplayView  # noqa: F401
import PreferencesPanelController  # noqa: F401
from PyObjCTools import AppHelper

# start the event loop
AppHelper.runEventLoop()



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level, min_sdk_level
import objc


class TestNSTextSelectionNavigationHelper(AppKit.NSObject):
    def locationFromLocation_withOffset_(self, a, b):
        return 1

    def offsetFromLocation_toLocation_(self, a, b):
        return 1

    def baseWritingDirectionAtLocation_(self, a):
        return 1

    def enumerateCaretOffsetsInLineFragmentAtLocation_usingBlock_(self, a, b):
        pass

    def lineFragmentRangeForPoint_inContainerAtLocation_(self, a, b):
        return 1

    def enumerateContainerBoundariesFromLocation_reverse_usingBlock_(self, a, b, c):
        pass

    def enumerateSubstringsFromLocation_options_usingBlock_(self, a, b, c):
        pass

    def textLayoutOrientationAtLocation_(self, a):
        return 1


class TestNSTextSelectionNavigation(TestCase):
    def test_constants(self):
        self.assertEqual(AppKit.NSTextSelectionNavigationDirectionForward, 0)
        self.assertEqual(AppKit.NSTextSelectionNavigationDirectionBackward, 1)
        self.assertEqual(AppKit.NSTextSelectionNavigationDirectionRight, 2)
        self.assertEqual(AppKit.NSTextSelectionNavigationDirectionLeft, 3)
        self.assertEqual(AppKit.NSTextSelectionNavigationDirectionUp, 4)
        self.assertEqual(AppKit.NSTextSelectionNavigationDirectionDown, 5)

        self.assertEqual(AppKit.NSTextSelectionNavigationDestinationCharacter, 0)
        self.assertEqual(AppKit.NSTextSelectionNavigationDestinationWord, 1)
        self.assertEqual(AppKit.NSTextSelectionNavigationDestinationLine, 2)
        self.assertEqual(AppKit.NSTextSelectionNavigationDestinationSentence, 3)
        self.assertEqual(AppKit.NSTextSelectionNavigationDestinationParagraph, 4)
        self.assertEqual(AppKit.NSTextSelectionNavigationDestinationContainer, 5)
        self.assertEqual(AppKit.NSTextSelectionNavigationDestinationDocument, 6)

        self.assertEqual(AppKit.NSTextSelectionNavigationModifierExtend, 1 << 0)
        self.assertEqual(AppKit.NSTextSelectionNavigationModifierVisual, 1 << 1)
        self.assertEqual(AppKit.NSTextSelectionNavigationModifierMultiple, 1 << 2)

        self.assertEqual(AppKit.NSTextSelectionNavigationWritingDirectionLeftToRight, 0)
        self.assertEqual(AppKit.NSTextSelectionNavigationWritingDirectionRightToLeft, 1)

        self.assertEqual(AppKit.NSTextSelectionNavigationLayoutOrientationHorizontal, 0)
        self.assertEqual(AppKit.NSTextSelectionNavigationLayoutOrientationVertical, 1)

    @min_sdk_level("12.0")
    def test_protocols(self):
        objc.protocolNamed("NSTextSelectionDataSource")

    def test_methods(self):
        self.assertArgHasType(
            TestNSTextSelectionNavigationHelper.locationFromLocation_withOffset_,
            1,
            objc._C_NSInteger,
        )
        self.assertResultHasType(
            TestNSTextSelectionNavigationHelper.offsetFromLocation_toLocation_,
            objc._C_NSInteger,
        )
        self.assertResultHasType(
            TestNSTextSelectionNavigationHelper.baseWritingDirectionAtLocation_,
            objc._C_NSInteger,
        )

        self.assertArgIsBlock(
            TestNSTextSelectionNavigationHelper.enumerateCaretOffsetsInLineFragmentAtLocation_usingBlock_,
            1,
            b"vd@Zo^Z",
        )

        self.assertArgHasType(
            TestNSTextSelectionNavigationHelper.lineFragmentRangeForPoint_inContainerAtLocation_,
            0,
            AppKit.NSPoint.__typestr__,
        )

        self.assertArgIsBOOL(
            TestNSTextSelectionNavigationHelper.enumerateContainerBoundariesFromLocation_reverse_usingBlock_,
            1,
        )
        self.assertArgIsBlock(
            TestNSTextSelectionNavigationHelper.enumerateContainerBoundariesFromLocation_reverse_usingBlock_,
            2,
            b"v@o^Z",
        )

        self.assertArgIsBlock(
            TestNSTextSelectionNavigationHelper.enumerateSubstringsFromLocation_options_usingBlock_,
            2,
            b"v@@@o^Z",
        )

        self.assertArgIsBOOL(
            TestNSTextSelectionNavigationHelper.enumerateContainerBoundariesFromLocation_reverse_usingBlock_,
            1,
        )
        self.assertArgIsBlock(
            TestNSTextSelectionNavigationHelper.enumerateContainerBoundariesFromLocation_reverse_usingBlock_,
            2,
            b"v@o^Z",
        )

        self.assertResultHasType(
            TestNSTextSelectionNavigationHelper.textLayoutOrientationAtLocation_,
            objc._C_NSInteger,
        )

    @min_os_level("12.0")
    def test_methods12_0(self):
        self.assertResultIsBOOL(
            AppKit.NSTextSelectionNavigation.allowsNonContiguousRanges
        )
        self.assertArgIsBOOL(
            AppKit.NSTextSelectionNavigation.setAllowsNonContiguousRanges_, 0
        )

        self.assertResultIsBOOL(
            AppKit.NSTextSelectionNavigation.rotatesCoordinateSystemForLayoutOrientation
        )
        self.assertArgIsBOOL(
            AppKit.NSTextSelectionNavigation.setRotatesCoordinateSystemForLayoutOrientation_,
            0,
        )

        self.assertArgIsBOOL(
            AppKit.NSTextSelectionNavigation.destinationSelectionForTextSelection_direction_destination_extending_confined_,
            3,
        )
        self.assertArgIsBOOL(
            AppKit.NSTextSelectionNavigation.destinationSelectionForTextSelection_direction_destination_extending_confined_,
            4,
        )
        self.assertArgIsBOOL(
            AppKit.NSTextSelectionNavigation.textSelectionsInteractingAtPoint_inContainerAtLocation_anchors_modifiers_selecting_bounds_,  # noqa: B950
            4,
        )
        self.assertArgIsBOOL(
            AppKit.NSTextSelectionNavigation.deletionRangesForTextSelection_direction_destination_allowsDecomposition_,
            3,
        )



import Foundation
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSJSONSerialization(TestCase):
    @min_os_level("10.7")
    def testConstants10_7(self):
        self.assertEqual(Foundation.NSJSONReadingMutableContainers, (1 << 0))
        self.assertEqual(Foundation.NSJSONReadingMutableLeaves, (1 << 1))
        self.assertEqual(Foundation.NSJSONReadingAllowFragments, (1 << 2))
        self.assertEqual(Foundation.NSJSONReadingFragmentsAllowed, (1 << 2))
        self.assertEqual(Foundation.NSJSONReadingJSON5Allowed, (1 << 3))
        self.assertEqual(Foundation.NSJSONReadingTopLevelDictionaryAssumed, (1 << 4))

        self.assertEqual(Foundation.NSJSONWritingPrettyPrinted, (1 << 0))
        self.assertEqual(Foundation.NSJSONWritingSortedKeys, (1 << 1))
        self.assertEqual(Foundation.NSJSONWritingFragmentsAllowed, (1 << 2))
        self.assertEqual(Foundation.NSJSONWritingWithoutEscapingSlashes, (1 << 3))

    @min_os_level("10.7")
    def testMethod10_7(self):
        self.assertResultIsBOOL(Foundation.NSJSONSerialization.isValidJSONObject_)
        self.assertArgIsOut(
            Foundation.NSJSONSerialization.dataWithJSONObject_options_error_, 2
        )
        self.assertArgIsOut(
            Foundation.NSJSONSerialization.JSONObjectWithData_options_error_, 2
        )
        self.assertArgIsOut(
            Foundation.NSJSONSerialization.writeJSONObject_toStream_options_error_, 3
        )
        self.assertArgIsOut(
            Foundation.NSJSONSerialization.JSONObjectWithStream_options_error_, 2
        )



import Foundation
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSTimeZone(TestCase):
    def testMethods(self):
        self.assertResultIsBOOL(Foundation.NSTimeZone.isDaylightSavingTimeForDate_)
        self.assertResultIsBOOL(Foundation.NSTimeZone.isDaylightSavingTime)
        self.assertResultIsBOOL(Foundation.NSTimeZone.isEqualToTimeZone_)

    def testConstants(self):
        self.assertEqual(Foundation.NSTimeZoneNameStyleStandard, 0)
        self.assertEqual(Foundation.NSTimeZoneNameStyleShortStandard, 1)
        self.assertEqual(Foundation.NSTimeZoneNameStyleDaylightSaving, 2)
        self.assertEqual(Foundation.NSTimeZoneNameStyleShortDaylightSaving, 3)

        self.assertIsInstance(Foundation.NSSystemTimeZoneDidChangeNotification, str)

    @min_os_level("10.6")
    def testConstants10_6(self):
        self.assertEqual(Foundation.NSTimeZoneNameStyleGeneric, 4)
        self.assertEqual(Foundation.NSTimeZoneNameStyleShortGeneric, 5)



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSScreen(TestCase):
    def testMethods(self):
        m = AppKit.NSScreen.supportedWindowDepths.__metadata__()
        self.assertTrue(m["retval"]["c_array_delimited_by_null"])

    @min_os_level("10.6")
    def testConstants10_6(self):
        self.assertIsInstance(AppKit.NSScreenColorSpaceDidChangeNotification, str)

    @min_os_level("10.7")
    def testMethods10_7(self):
        self.assertResultHasType(
            AppKit.NSScreen.convertRectToBacking_, AppKit.NSRect.__typestr__
        )
        self.assertArgHasType(
            AppKit.NSScreen.convertRectToBacking_, 0, AppKit.NSRect.__typestr__
        )
        self.assertResultHasType(
            AppKit.NSScreen.convertRectFromBacking_, AppKit.NSRect.__typestr__
        )
        self.assertArgHasType(
            AppKit.NSScreen.convertRectFromBacking_, 0, AppKit.NSRect.__typestr__
        )
        self.assertResultHasType(
            AppKit.NSScreen.backingAlignedRect_options_, AppKit.NSRect.__typestr__
        )
        self.assertArgHasType(
            AppKit.NSScreen.backingAlignedRect_options_, 0, AppKit.NSRect.__typestr__
        )

    @min_os_level("10.9")
    def testMethods10_9(self):
        self.assertResultIsBOOL(AppKit.NSScreen.screensHaveSeparateSpaces)

    @min_os_level("10.12")
    def testMethods10_12(self):
        self.assertResultIsBOOL(AppKit.NSScreen.canRepresentDisplayGamut_)



import AppKit
from PyObjCTools.TestSupport import TestCase


class TestNSSegmentedCell(TestCase):
    def testConstants(self):
        self.assertEqual(AppKit.NSSegmentSwitchTrackingSelectOne, 0)
        self.assertEqual(AppKit.NSSegmentSwitchTrackingSelectAny, 1)
        self.assertEqual(AppKit.NSSegmentSwitchTrackingMomentary, 2)

    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSSegmentedCell.selectSegmentWithTag_)
        self.assertResultIsBOOL(AppKit.NSSegmentedCell.isSelectedForSegment_)
        self.assertArgIsBOOL(AppKit.NSSegmentedCell.setSelected_forSegment_, 0)
        self.assertResultIsBOOL(AppKit.NSSegmentedCell.isEnabledForSegment_)
        self.assertArgIsBOOL(AppKit.NSSegmentedCell.setEnabled_forSegment_, 0)



import AppKit
from PyObjCTools.TestSupport import (
    TestCase,
    min_os_level,
    min_sdk_level,
    expectedFailure,
)
import objc


class TestNSKeyValueBindingHelper(AppKit.NSObject):
    def commitEditingWithDelegate_didCommitSelector_contextInfo_(self, d, s, i):
        return None

    def commitEditingAndReturnError_(self, v):
        return 1

    def commitEditing(self):
        return 1


class TestNSKeyValueBinding(TestCase):
    def testConstants(self):
        self.assertIsInstance(AppKit.NSMultipleValuesMarker, AppKit.NSObject)
        self.assertIsInstance(AppKit.NSNoSelectionMarker, AppKit.NSObject)
        self.assertIsInstance(AppKit.NSNotApplicableMarker, AppKit.NSObject)

        self.assertIsInstance(AppKit.NSObservedObjectKey, str)
        self.assertIsInstance(AppKit.NSObservedKeyPathKey, str)
        self.assertIsInstance(AppKit.NSOptionsKey, str)

        self.assertIsInstance(AppKit.NSAlignmentBinding, str)
        self.assertIsInstance(AppKit.NSAlternateImageBinding, str)
        self.assertIsInstance(AppKit.NSAlternateTitleBinding, str)
        self.assertIsInstance(AppKit.NSAnimateBinding, str)
        self.assertIsInstance(AppKit.NSAnimationDelayBinding, str)
        self.assertIsInstance(AppKit.NSArgumentBinding, str)
        self.assertIsInstance(AppKit.NSAttributedStringBinding, str)
        self.assertIsInstance(AppKit.NSContentArrayBinding, str)
        self.assertIsInstance(AppKit.NSContentArrayForMultipleSelectionBinding, str)
        self.assertIsInstance(AppKit.NSContentBinding, str)
        self.assertIsInstance(AppKit.NSContentHeightBinding, str)
        self.assertIsInstance(AppKit.NSContentObjectBinding, str)
        self.assertIsInstance(AppKit.NSContentObjectsBinding, str)
        self.assertIsInstance(AppKit.NSContentSetBinding, str)
        self.assertIsInstance(AppKit.NSContentValuesBinding, str)
        self.assertIsInstance(AppKit.NSContentWidthBinding, str)
        self.assertIsInstance(AppKit.NSCriticalValueBinding, str)
        self.assertIsInstance(AppKit.NSDataBinding, str)
        self.assertIsInstance(AppKit.NSDisplayPatternTitleBinding, str)
        self.assertIsInstance(AppKit.NSDisplayPatternValueBinding, str)
        self.assertIsInstance(AppKit.NSDocumentEditedBinding, str)
        self.assertIsInstance(AppKit.NSDoubleClickArgumentBinding, str)
        self.assertIsInstance(AppKit.NSDoubleClickTargetBinding, str)
        self.assertIsInstance(AppKit.NSEditableBinding, str)
        self.assertIsInstance(AppKit.NSEnabledBinding, str)
        self.assertIsInstance(AppKit.NSFilterPredicateBinding, str)
        self.assertIsInstance(AppKit.NSFontBinding, str)
        self.assertIsInstance(AppKit.NSFontBoldBinding, str)
        self.assertIsInstance(AppKit.NSFontFamilyNameBinding, str)
        self.assertIsInstance(AppKit.NSFontItalicBinding, str)
        self.assertIsInstance(AppKit.NSFontNameBinding, str)
        self.assertIsInstance(AppKit.NSFontSizeBinding, str)
        self.assertIsInstance(AppKit.NSHeaderTitleBinding, str)
        self.assertIsInstance(AppKit.NSHiddenBinding, str)
        self.assertIsInstance(AppKit.NSImageBinding, str)
        self.assertIsInstance(AppKit.NSIsIndeterminateBinding, str)
        self.assertIsInstance(AppKit.NSLabelBinding, str)
        self.assertIsInstance(AppKit.NSManagedObjectContextBinding, str)
        self.assertIsInstance(AppKit.NSMaximumRecentsBinding, str)
        self.assertIsInstance(AppKit.NSMaxValueBinding, str)
        self.assertIsInstance(AppKit.NSMaxWidthBinding, str)
        self.assertIsInstance(AppKit.NSMinValueBinding, str)
        self.assertIsInstance(AppKit.NSMinWidthBinding, str)
        self.assertIsInstance(AppKit.NSMixedStateImageBinding, str)
        self.assertIsInstance(AppKit.NSOffStateImageBinding, str)
        self.assertIsInstance(AppKit.NSOnStateImageBinding, str)
        self.assertIsInstance(AppKit.NSPredicateBinding, str)
        self.assertIsInstance(AppKit.NSRecentSearchesBinding, str)
        self.assertIsInstance(AppKit.NSRepresentedFilenameBinding, str)
        self.assertIsInstance(AppKit.NSRowHeightBinding, str)
        self.assertIsInstance(AppKit.NSSelectedIdentifierBinding, str)
        self.assertIsInstance(AppKit.NSSelectedIndexBinding, str)
        self.assertIsInstance(AppKit.NSSelectedLabelBinding, str)
        self.assertIsInstance(AppKit.NSSelectedObjectBinding, str)
        self.assertIsInstance(AppKit.NSSelectedObjectsBinding, str)
        self.assertIsInstance(AppKit.NSSelectedTagBinding, str)
        self.assertIsInstance(AppKit.NSSelectedValueBinding, str)
        self.assertIsInstance(AppKit.NSSelectedValuesBinding, str)
        self.assertIsInstance(AppKit.NSSelectionIndexesBinding, str)
        self.assertIsInstance(AppKit.NSSelectionIndexPathsBinding, str)
        self.assertIsInstance(AppKit.NSSortDescriptorsBinding, str)
        self.assertIsInstance(AppKit.NSTargetBinding, str)
        self.assertIsInstance(AppKit.NSTextColorBinding, str)
        self.assertIsInstance(AppKit.NSTitleBinding, str)
        self.assertIsInstance(AppKit.NSToolTipBinding, str)
        self.assertIsInstance(AppKit.NSValueBinding, str)
        self.assertIsInstance(AppKit.NSValuePathBinding, str)
        self.assertIsInstance(AppKit.NSValueURLBinding, str)
        self.assertIsInstance(AppKit.NSVisibleBinding, str)
        self.assertIsInstance(AppKit.NSWarningValueBinding, str)
        self.assertIsInstance(AppKit.NSWidthBinding, str)

        self.assertIsInstance(
            AppKit.NSAllowsEditingMultipleValuesSelectionBindingOption, str
        )
        self.assertIsInstance(AppKit.NSAllowsNullArgumentBindingOption, str)
        self.assertIsInstance(
            AppKit.NSAlwaysPresentsApplicationModalAlertsBindingOption, str
        )
        self.assertIsInstance(AppKit.NSConditionallySetsEditableBindingOption, str)
        self.assertIsInstance(AppKit.NSConditionallySetsEnabledBindingOption, str)
        self.assertIsInstance(AppKit.NSConditionallySetsHiddenBindingOption, str)
        self.assertIsInstance(AppKit.NSContinuouslyUpdatesValueBindingOption, str)
        self.assertIsInstance(AppKit.NSCreatesSortDescriptorBindingOption, str)
        self.assertIsInstance(AppKit.NSDeletesObjectsOnRemoveBindingsOption, str)
        self.assertIsInstance(AppKit.NSDisplayNameBindingOption, str)
        self.assertIsInstance(AppKit.NSDisplayPatternBindingOption, str)
        self.assertIsInstance(AppKit.NSHandlesContentAsCompoundValueBindingOption, str)
        self.assertIsInstance(AppKit.NSInsertsNullPlaceholderBindingOption, str)
        self.assertIsInstance(
            AppKit.NSInvokesSeparatelyWithArrayObjectsBindingOption, str
        )
        self.assertIsInstance(AppKit.NSMultipleValuesPlaceholderBindingOption, str)
        self.assertIsInstance(AppKit.NSNoSelectionPlaceholderBindingOption, str)
        self.assertIsInstance(AppKit.NSNotApplicablePlaceholderBindingOption, str)
        self.assertIsInstance(AppKit.NSNullPlaceholderBindingOption, str)
        self.assertIsInstance(AppKit.NSRaisesForNotApplicableKeysBindingOption, str)
        self.assertIsInstance(AppKit.NSPredicateFormatBindingOption, str)
        self.assertIsInstance(AppKit.NSSelectorNameBindingOption, str)
        self.assertIsInstance(AppKit.NSSelectsAllWhenSettingContentBindingOption, str)
        self.assertIsInstance(AppKit.NSValidatesImmediatelyBindingOption, str)
        self.assertIsInstance(AppKit.NSValueTransformerNameBindingOption, str)
        self.assertIsInstance(AppKit.NSValueTransformerBindingOption, str)

    @min_os_level("10.5")
    def testConstants10_5(self):
        self.assertIsInstance(AppKit.NSContentDictionaryBinding, str)
        self.assertIsInstance(AppKit.NSExcludedKeysBinding, str)
        self.assertIsInstance(AppKit.NSIncludedKeysBinding, str)
        self.assertIsInstance(AppKit.NSInitialKeyBinding, str)
        self.assertIsInstance(AppKit.NSInitialValueBinding, str)
        self.assertIsInstance(AppKit.NSLocalizedKeyDictionaryBinding, str)
        self.assertIsInstance(AppKit.NSTransparentBinding, str)
        self.assertIsInstance(AppKit.NSContentPlacementTagBindingOption, str)

    @min_os_level("10.7")
    def testConstants10_7(self):
        self.assertIsInstance(AppKit.NSPositioningRectBinding, str)

    def testFunctions(self):
        o = AppKit.NSObject.alloc().init()
        self.assertIs(AppKit.NSIsControllerMarker(o), False)
        self.assertIs(AppKit.NSIsControllerMarker(AppKit.NSMultipleValuesMarker), True)

    def testMethods(self):
        o = TestNSKeyValueBindingHelper.alloc().init()
        m = o.commitEditingWithDelegate_didCommitSelector_contextInfo_.__metadata__()
        self.assertEqual(m["arguments"][3]["sel_of_type"], b"v@:@Z^v")

        self.assertResultIsBOOL(TestNSKeyValueBindingHelper.commitEditing)

    @min_os_level("10.7")
    @expectedFailure
    def testMethods10_7(self):
        self.assertResultIsBOOL(
            TestNSKeyValueBindingHelper.commitEditingAndReturnError_
        )
        self.assertArgIsOut(TestNSKeyValueBindingHelper.commitEditingAndReturnError_, 0)

    @min_sdk_level("10.14")
    def test_protocols(self):
        objc.protocolNamed("NSEditor")
        objc.protocolNamed("NSEditorRegistration")



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSTextInputContext(TestCase):
    @min_os_level("10.6")
    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSTextInputContext.acceptsGlyphInfo)
        self.assertArgIsBOOL(AppKit.NSTextInputContext.setAcceptsGlyphInfo_, 0)
        self.assertResultIsBOOL(AppKit.NSTextInputContext.handleEvent_)

    @min_os_level("10.6")
    def testConstants(self):
        self.assertIsInstance(
            AppKit.NSTextInputContextKeyboardSelectionDidChangeNotification, str
        )



"""
Script for building the example.

Usage:
    python3 setup.py py2app
"""
import glob

from setuptools import setup

images = glob.glob("Images/*.tiff")
icons = glob.glob("Icons/*.icns")

plist = {
    "CFBundleShortVersionString": "To Do v1",
    "CFBundleIconFile": "ToDoApp.icns",
    "CFBundleGetInfoString": "To Do v1",
    "CFBundleIdentifier": "net.sf.pyobjc.ToDo",
    "CFBundleDocumentTypes": [
        {
            "CFBundleTypeName": "To Do list",
            "CFBundleTypeRole": "Editor",
            "NSDocumentClass": "ToDoDocument",
            "CFBundleTypeIconFile": "ToDoDoc.icns",
            "CFBundleTypeExtensions": ["ToDo"],
            "CFBundleTypeOSTypes": ["ToDo"],
        }
    ],
    "CFBundleName": "To Do",
}

setup(
    app=["main.py"],
    data_files=["English.lproj"] + images + icons,
    options={"py2app": {"plist": plist}},
    setup_requires=["py2app", "pyobjc-framework-Cocoa"],
)



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSColorPickerTouchBarItem(TestCase):
    @min_os_level("10.12")
    def testMethods10_12(self):
        self.assertResultIsBOOL(AppKit.NSColorPickerTouchBarItem.showsAlpha)
        self.assertArgIsBOOL(AppKit.NSColorPickerTouchBarItem.setShowsAlpha_, 0)

        self.assertResultIsBOOL(AppKit.NSColorPickerTouchBarItem.isEnabled)
        self.assertArgIsBOOL(AppKit.NSColorPickerTouchBarItem.setEnabled_, 0)



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level, min_sdk_level
import objc


class TestNSResponder(TestCase):
    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSResponder.tryToPerform_with_)
        self.assertResultIsBOOL(AppKit.NSResponder.performKeyEquivalent_)
        self.assertResultIsBOOL(AppKit.NSResponder.acceptsFirstResponder)
        self.assertResultIsBOOL(AppKit.NSResponder.becomeFirstResponder)
        self.assertResultIsBOOL(AppKit.NSResponder.resignFirstResponder)
        self.assertResultIsBOOL(AppKit.NSResponder.shouldBeTreatedAsInkEvent_)
        self.assertResultIsBOOL(AppKit.NSResponder.performMnemonic_)
        self.assertArgIsSEL(AppKit.NSResponder.doCommandBySelector_, 0, b"v@:@")

        self.assertArgIsSEL(
            AppKit.NSResponder.presentError_modalForWindow_delegate_didPresentSelector_contextInfo_,  # noqa: B950
            3,
            b"v@:" + objc._C_NSBOOL + b"^v",
        )
        self.assertArgHasType(
            AppKit.NSResponder.presentError_modalForWindow_delegate_didPresentSelector_contextInfo_,  # noqa: B950
            4,
            b"^v",
        )
        self.assertResultIsBOOL(AppKit.NSResponder.presentError_)

    @min_os_level("10.7")
    def testMethods10_7(self):
        self.assertResultIsBOOL(
            AppKit.NSResponder.wantsScrollEventsForSwipeTrackingOnAxis_
        )
        self.assertResultIsBOOL(AppKit.NSResponder.wantsForwardedScrollEventsForAxis_)
        self.assertArgIsSEL(
            AppKit.NSResponder.supplementalTargetForAction_sender_, 0, b"v@:@"
        )
        self.assertResultIsBOOL(
            AppKit.NSResponder.validateProposedFirstResponder_forEvent_
        )

    @min_sdk_level("10.14")
    def testProtocols(self):
        objc.protocolNamed("NSStandardKeyBindingResponding")



from math import cos, pi, sin

import Cocoa
import objc
from fieldMath import degToRad
from objc import super

# Convenience global variables
x, y = 0, 1
llc, sze = 0, 1  # Left Lower Corner, Size

BLACK = Cocoa.NSColor.blackColor()
BLUE = Cocoa.NSColor.blueColor()
GREEN = Cocoa.NSColor.greenColor()


class CGraphView(Cocoa.NSView):

    azmuthSlider = objc.IBOutlet()
    mapOffsetEWSlider = objc.IBOutlet()
    mapOffsetNSSlider = objc.IBOutlet()
    mapScaleSlider = objc.IBOutlet()
    mapVisibleSlider = objc.IBOutlet()
    azmuthDisplay = objc.IBOutlet()
    mapOffsetEWDisplay = objc.IBOutlet()
    mapOffsetNSDisplay = objc.IBOutlet()
    mapScaleDisplay = objc.IBOutlet()

    def initWithFrame_(self, frame):
        super().initWithFrame_(frame)
        self.setGridColor()
        self.setRmsColor()
        self.setGraphColor()
        self.graphMargin = 2
        self.mapImage = 0
        self.mapRect = 0
        self.mapVisible = 0.70
        self.mapScale = 3.0
        self.mapOffsetEW = 0.27
        self.mapOffsetNS = 0.40
        self.mapBaseRadius = 200

        self.lines = 2
        self.gain = 0.5
        return self

    def awakeFromNib(self):
        self.setCrossCursor()
        self.mapVisibleSlider.setFloatValue_(self.mapVisible)
        self.setAzmuth_(125)
        self.setMapRect()

    @objc.python_method
    def setCrossCursor(self):
        crosshairImage = Cocoa.NSImage.imageNamed_("CrossCursor")
        self.crossCursor = Cocoa.NSCursor.alloc().initWithImage_hotSpot_(
            crosshairImage, (8, 8)
        )
        self.trackingRect = self.addTrackingRect_owner_userData_assumeInside_(
            self.bounds(), self, 0, 0
        )

    @objc.python_method
    def setGridColor(self, color=GREEN):
        self.gridColor = color

    @objc.python_method
    def setRmsColor(self, color=BLUE):
        self.rmsColor = color

    @objc.python_method
    def setGraphColor(self, color=BLACK):
        self.graphColor = color

    @objc.python_method
    def setGain(self, gain, total):
        self.gain = gain
        self.totalField = total

    @objc.python_method
    def setLines(self, lines):
        self.lines = lines

    @objc.python_method
    def setMapImage(self, mapImage):
        self.mapImage = mapImage
        self.mapRect = ((0, 0), mapImage.size())

    @objc.python_method
    def setPath(self, path, maxMag):
        self.path = path
        self.maxMag = maxMag
        self.setNeedsDisplay_(1)

    def drawRect_(self, rect):
        frame = self.frame()
        self.origin = frame[0]
        self.graphCenter = (frame[sze][x] / 2, frame[sze][y] / 2)
        self.graphRadius = (min(frame[sze][x], frame[sze][y]) / 2) - self.graphMargin

        Cocoa.NSColor.whiteColor().set()
        Cocoa.NSRectFill(self.bounds())

        self.drawMap()
        self.drawGrid()
        self.drawRMS()
        self.drawField()

    def drawMap(self):
        if self.mapImage == 0:
            return

        scale = (
            self.mapScale
            * (self.graphRadius / self.mapBaseRadius)
            * self.gain
            / self.totalField
        )
        xImageSize = scale * self.mapRect[sze][x]
        yImageSize = scale * self.mapRect[sze][y]
        xCenterMove = self.graphCenter[x] - self.graphRadius
        yCenterMove = self.graphCenter[y] - self.graphRadius

        xOffset = -((1 - self.mapOffsetEW) / 2) * xImageSize
        yOffset = -((1 + self.mapOffsetNS) / 2) * yImageSize
        xOffset += self.graphRadius + xCenterMove
        yOffset += self.graphRadius + yCenterMove

        drawInRect = ((xOffset, yOffset), (xImageSize, yImageSize))

        self.mapImage.drawInRect_fromRect_operation_fraction_(
            drawInRect, self.mapRect, Cocoa.NSCompositeSourceOver, self.mapVisible
        )

    def drawGrid(self):
        self.gridColor.set()
        self.drawCircle_(1.0)
        self.drawAxisLines()

    def drawCircle_(self, scale):
        center = self.graphCenter
        radius = self.graphRadius * scale
        x, y = 0, 1
        if radius >= 1:
            dotRect = (
                (center[x] - radius, center[y] - radius),
                (2 * radius, 2 * radius),
            )
            path = Cocoa.NSBezierPath.bezierPathWithOvalInRect_(dotRect)
            path.stroke()

    def drawRMS(self):
        self.rmsColor.set()
        self.drawCircle_(self.gain)

    def drawAxisLines(self):
        center = self.graphCenter
        radius = self.graphRadius
        x, y = 0, 1
        path = Cocoa.NSBezierPath.bezierPath()
        for i in range(1, self.lines + 1):
            iR = pi / i
            cosR = cos(iR) * radius
            sinR = sin(iR) * radius

            path.moveToPoint_((center[x] - cosR, center[y] - sinR))
            path.lineToPoint_((center[x] + cosR, center[y] + sinR))
        path.closePath()
        path.stroke()

    def drawField(self):
        if self.maxMag:  # Don't want to divide by zero in the pathological case
            self.graphColor.set()
            path = self.path.copy()

            transform = Cocoa.NSAffineTransform.transform()
            transform.rotateByRadians_(-(pi / 2.0) - self.azmuth)
            path.transformUsingAffineTransform_(transform)

            transform = Cocoa.NSAffineTransform.transform()
            center = self.graphCenter
            transform.translateXBy_yBy_(center[0], center[1])
            transform.scaleBy_(self.graphRadius / self.maxMag)
            path.transformUsingAffineTransform_(transform)

            path.stroke()

    # ____________________________________________________________
    # Handle GUI values
    @objc.IBAction
    def mapVisibleSlider_(self, sender):
        self.mapVisible = sender.floatValue()
        self.setNeedsDisplay_(1)

    @objc.IBAction
    def azmuthDisplay_(self, sender):
        self.setAzmuth_(sender.floatValue())

    @objc.IBAction
    def azmuthSlider_(self, sender):
        self.setAzmuth_(sender.floatValue())

    def setAzmuth_(self, value):
        self.azmuth = degToRad(value)
        self.azmuthSlider.setFloatValue_(value)
        self.azmuthDisplay.setFloatValue_(value)
        self.setNeedsDisplay_(1)

    @objc.IBAction
    def mapScaleDisplay_(self, sender):
        self.mapScale = sender.floatValue()
        self.setMapRect()

    @objc.IBAction
    def mapScaleSlider_(self, sender):
        self.mapScale = sender.floatValue()
        self.setMapRect()

    @objc.IBAction
    def mapOffsetNSDisplay_(self, sender):
        self.mapOffsetNS = sender.floatValue()
        self.setMapRect()

    @objc.IBAction
    def mapOffsetNSSlider_(self, sender):
        self.mapOffsetNS = sender.floatValue()
        self.setMapRect()

    @objc.IBAction
    def mapOffsetEWDisplay_(self, sender):
        self.mapOffsetEW = sender.floatValue()
        self.setMapRect()

    @objc.IBAction
    def mapOffsetEWSlider_(self, sender):
        self.mapOffsetEW = sender.floatValue()
        self.setMapRect()

    def mouseUp_(self, event):
        loc = event.locationInWindow()
        xLoc = loc[x] - self.origin[x]
        yLoc = loc[y] - self.origin[y]
        xDelta = self.graphCenter[x] - xLoc
        yDelta = self.graphCenter[y] - yLoc

        scale = (
            0.5
            * self.mapScale
            * (self.gain / self.totalField)
            * (self.graphRadius / self.mapBaseRadius)
        )
        xOffset = xDelta / (scale * self.mapRect[sze][x])
        yOffset = yDelta / (scale * self.mapRect[sze][y])

        self.mapOffsetEW += xOffset
        self.mapOffsetNS -= yOffset
        self.setMapRect()

    def mouseDown_(self, event):
        self.crossCursor.set()

    def setMapRect(self):
        self.mapScaleDisplay.setFloatValue_(self.mapScale)
        self.mapScaleSlider.setFloatValue_(self.mapScale)
        self.mapOffsetEWDisplay.setFloatValue_(self.mapOffsetEW)
        self.mapOffsetEWSlider.setFloatValue_(self.mapOffsetEW)
        self.mapOffsetNSDisplay.setFloatValue_(self.mapOffsetNS)
        self.mapOffsetNSSlider.setFloatValue_(self.mapOffsetNS)
        self.setNeedsDisplay_(1)

    def mouseEntered_(self, event):
        print("CGraphView: mouseEntered_")

    def mouseExited_(self, event):
        print("CGraphView: mouseExited_")



import Foundation
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSHTTPCookie(TestCase):
    def testConstants(self):
        self.assertIsInstance(Foundation.NSHTTPCookieName, str)
        self.assertIsInstance(Foundation.NSHTTPCookieValue, str)
        self.assertIsInstance(Foundation.NSHTTPCookieOriginURL, str)
        self.assertIsInstance(Foundation.NSHTTPCookieVersion, str)
        self.assertIsInstance(Foundation.NSHTTPCookieDomain, str)
        self.assertIsInstance(Foundation.NSHTTPCookiePath, str)
        self.assertIsInstance(Foundation.NSHTTPCookieSecure, str)
        self.assertIsInstance(Foundation.NSHTTPCookieExpires, str)
        self.assertIsInstance(Foundation.NSHTTPCookieComment, str)
        self.assertIsInstance(Foundation.NSHTTPCookieCommentURL, str)
        self.assertIsInstance(Foundation.NSHTTPCookieDiscard, str)
        self.assertIsInstance(Foundation.NSHTTPCookieMaximumAge, str)
        self.assertIsInstance(Foundation.NSHTTPCookiePort, str)

    @min_os_level("10.15")
    def testConstants10_15(self):
        self.assertIsInstance(Foundation.NSHTTPCookieSameSitePolicy, str)
        self.assertIsInstance(Foundation.NSHTTPCookieSameSiteLax, str)
        self.assertIsInstance(Foundation.NSHTTPCookieSameSiteStrict, str)

    def testMethods(self):
        self.assertResultIsBOOL(Foundation.NSHTTPCookie.isSessionOnly)
        self.assertResultIsBOOL(Foundation.NSHTTPCookie.isSecure)
        self.assertResultIsBOOL(Foundation.NSHTTPCookie.isHTTPOnly)



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level, min_sdk_level
import objc


class TestNSImageHelper(AppKit.NSObject):
    def image_didLoadRepresentation_withStatus_(self, i, r, s):
        pass

    def image_didLoadPartOfRepresentation_withValidRows_(self, i, r, c):
        pass


class TestNSImage(TestCase):
    def test_compositePoint(self):
        # comes straight from ReSTedit.  Works on PPC, not on Intel (as of r1791)
        ws = AppKit.NSWorkspace.sharedWorkspace()
        txtIcon = ws.iconForFileType_("txt")
        txtIcon.setSize_((16, 16))
        htmlIcon = ws.iconForFileType_("html")
        htmlIcon.setSize_((16, 16))

        comboIcon = AppKit.NSImage.alloc().initWithSize_((100, 100))
        comboIcon.lockFocus()
        txtIcon.compositeToPoint_fromRect_operation_(
            (0, 0), ((0, 0), (16, 16)), AppKit.NSCompositeCopy
        )
        htmlIcon.compositeToPoint_fromRect_operation_(
            (8, 0), ((8, 0), (8, 16)), AppKit.NSCompositeCopy
        )
        comboIcon.unlockFocus()

    def testConstants(self):
        self.assertEqual(AppKit.NSImageLoadStatusCompleted, 0)
        self.assertEqual(AppKit.NSImageLoadStatusCancelled, 1)
        self.assertEqual(AppKit.NSImageLoadStatusInvalidData, 2)
        self.assertEqual(AppKit.NSImageLoadStatusUnexpectedEOF, 3)
        self.assertEqual(AppKit.NSImageLoadStatusReadError, 4)

        self.assertEqual(AppKit.NSImageCacheDefault, 0)
        self.assertEqual(AppKit.NSImageCacheAlways, 1)
        self.assertEqual(AppKit.NSImageCacheBySize, 2)
        self.assertEqual(AppKit.NSImageCacheNever, 3)

        if objc.arch == "x86_64":
            self.assertEqual(AppKit.NSImageResizingModeStretch, 0)
            self.assertEqual(AppKit.NSImageResizingModeTile, 1)
        else:
            self.assertEqual(AppKit.NSImageResizingModeTile, 0)
            self.assertEqual(AppKit.NSImageResizingModeStretch, 1)

        self.assertEqual(AppKit.NSImageSymbolScaleSmall, 1)
        self.assertEqual(AppKit.NSImageSymbolScaleMedium, 2)
        self.assertEqual(AppKit.NSImageSymbolScaleLarge, 3)

    @min_os_level("10.5")
    def testConstants10_5(self):
        self.assertIsInstance(AppKit.NSImageNameQuickLookTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameBluetoothTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameIChatTheaterTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameSlideshowTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameActionTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameSmartBadgeTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameIconViewTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameListViewTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameColumnViewTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameFlowViewTemplate, str)
        self.assertIsInstance(AppKit.NSImageNamePathTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameInvalidDataFreestandingTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameLockLockedTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameLockUnlockedTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameGoRightTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameGoLeftTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameRightFacingTriangleTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameLeftFacingTriangleTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameAddTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameRemoveTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameRevealFreestandingTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameFollowLinkFreestandingTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameEnterFullScreenTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameExitFullScreenTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameStopProgressTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameStopProgressFreestandingTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameRefreshTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameRefreshFreestandingTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameBonjour, str)
        self.assertIsInstance(AppKit.NSImageNameDotMac, str)
        self.assertIsInstance(AppKit.NSImageNameComputer, str)
        self.assertIsInstance(AppKit.NSImageNameFolderBurnable, str)
        self.assertIsInstance(AppKit.NSImageNameFolderSmart, str)
        self.assertIsInstance(AppKit.NSImageNameNetwork, str)
        self.assertIsInstance(AppKit.NSImageNameMultipleDocuments, str)
        self.assertIsInstance(AppKit.NSImageNameUserAccounts, str)
        self.assertIsInstance(AppKit.NSImageNamePreferencesGeneral, str)
        self.assertIsInstance(AppKit.NSImageNameAdvanced, str)
        self.assertIsInstance(AppKit.NSImageNameInfo, str)
        self.assertIsInstance(AppKit.NSImageNameFontPanel, str)
        self.assertIsInstance(AppKit.NSImageNameColorPanel, str)
        self.assertIsInstance(AppKit.NSImageNameUser, str)
        self.assertIsInstance(AppKit.NSImageNameUserGroup, str)
        self.assertIsInstance(AppKit.NSImageNameEveryone, str)

    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSImage.setName_)
        self.assertArgIsBOOL(AppKit.NSImage.setScalesWhenResized_, 0)
        self.assertResultIsBOOL(AppKit.NSImage.scalesWhenResized)
        self.assertArgIsBOOL(AppKit.NSImage.setDataRetained_, 0)
        self.assertResultIsBOOL(AppKit.NSImage.isDataRetained)
        self.assertArgIsBOOL(AppKit.NSImage.setCachedSeparately_, 0)
        self.assertResultIsBOOL(AppKit.NSImage.isCachedSeparately)
        self.assertArgIsBOOL(AppKit.NSImage.setCacheDepthMatchesImageDepth_, 0)
        self.assertResultIsBOOL(AppKit.NSImage.cacheDepthMatchesImageDepth)
        self.assertArgIsBOOL(AppKit.NSImage.setUsesEPSOnResolutionMismatch_, 0)
        self.assertResultIsBOOL(AppKit.NSImage.usesEPSOnResolutionMismatch)
        self.assertArgIsBOOL(AppKit.NSImage.setPrefersColorMatch_, 0)
        self.assertResultIsBOOL(AppKit.NSImage.prefersColorMatch)
        self.assertArgIsBOOL(AppKit.NSImage.setMatchesOnMultipleResolution_, 0)
        self.assertResultIsBOOL(AppKit.NSImage.matchesOnMultipleResolution)
        self.assertResultIsBOOL(AppKit.NSImage.drawRepresentation_inRect_)
        self.assertResultIsBOOL(AppKit.NSImage.isValid)
        self.assertResultIsBOOL(AppKit.NSImage.canInitWithPasteboard_)
        self.assertResultIsBOOL(AppKit.NSImage.isFlipped)
        self.assertArgIsBOOL(AppKit.NSImage.setFlipped_, 0)
        self.assertResultIsBOOL(AppKit.NSImage.isTemplate)
        self.assertArgIsBOOL(AppKit.NSImage.setTemplate_, 0)

    def testProtocols(self):

        self.assertArgHasType(
            TestNSImageHelper.image_didLoadPartOfRepresentation_withValidRows_,
            2,
            objc._C_NSInteger,
        )
        self.assertArgHasType(
            TestNSImageHelper.image_didLoadRepresentation_withStatus_,
            2,
            objc._C_NSUInteger,
        )

    @min_sdk_level("10.10")
    def testProtocolObjects(self):
        objc.protocolNamed("NSImageDelegate")

    @min_os_level("10.6")
    def testMethods10_6(self):
        self.assertArgHasType(
            AppKit.NSImage.drawInRect_fromRect_operation_fraction_respectFlipped_hints_,
            0,
            AppKit.NSRect.__typestr__,
        )
        self.assertArgIsBOOL(
            AppKit.NSImage.drawInRect_fromRect_operation_fraction_respectFlipped_hints_,
            4,
        )
        self.assertArgIsBOOL(AppKit.NSImage.lockFocusFlipped_, 0)
        self.assertArgHasType(
            AppKit.NSImage.initWithCGImage_size_, 1, AppKit.NSSize.__typestr__
        )
        self.assertArgHasType(
            AppKit.NSImage.CGImageForProposedRect_context_hints_,
            0,
            b"o^" + AppKit.NSRect.__typestr__,
        )
        self.assertArgHasType(
            AppKit.NSImage.bestRepresentationForRect_context_hints_,
            0,
            AppKit.NSRect.__typestr__,
        )

        self.assertResultIsBOOL(
            AppKit.NSImage.hitTestRect_withImageDestinationRect_context_hints_flipped_
        )
        self.assertArgHasType(
            AppKit.NSImage.hitTestRect_withImageDestinationRect_context_hints_flipped_,
            0,
            AppKit.NSRect.__typestr__,
        )
        self.assertArgHasType(
            AppKit.NSImage.hitTestRect_withImageDestinationRect_context_hints_flipped_,
            1,
            AppKit.NSRect.__typestr__,
        )

    @min_os_level("10.7")
    def testMethods10_7(self):
        self.assertResultIsBOOL(AppKit.NSImage.matchesOnlyOnBestFittingAxis)
        self.assertArgIsBOOL(AppKit.NSImage.setMatchesOnlyOnBestFittingAxis_, 0)

    @min_os_level("10.8")
    def testMethods10_8(self):
        self.assertArgIsBOOL(AppKit.NSImage.imageWithSize_flipped_drawingHandler_, 1)
        self.assertArgIsBlock(
            AppKit.NSImage.imageWithSize_flipped_drawingHandler_,
            2,
            objc._C_NSBOOL + AppKit.NSRect.__typestr__,
        )

    @min_os_level("10.6")
    def testConstants10_6(self):
        self.assertIsInstance(AppKit.NSImageHintCTM, str)
        self.assertIsInstance(AppKit.NSImageHintInterpolation, str)
        self.assertIsInstance(AppKit.NSImageNameFolder, str)
        self.assertIsInstance(AppKit.NSImageNameMobileMe, str)
        self.assertIsInstance(AppKit.NSImageNameUserGuest, str)
        self.assertIsInstance(AppKit.NSImageNameMenuOnStateTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameMenuMixedStateTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameApplicationIcon, str)
        self.assertIsInstance(AppKit.NSImageNameTrashEmpty, str)
        self.assertIsInstance(AppKit.NSImageNameTrashFull, str)
        self.assertIsInstance(AppKit.NSImageNameHomeTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameBookmarksTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameCaution, str)
        self.assertIsInstance(AppKit.NSImageNameStatusAvailable, str)
        self.assertIsInstance(AppKit.NSImageNameStatusPartiallyAvailable, str)
        self.assertIsInstance(AppKit.NSImageNameStatusUnavailable, str)
        self.assertIsInstance(AppKit.NSImageNameStatusNone, str)

    @min_os_level("10.8")
    def testConstants10_8(self):
        self.assertIsInstance(AppKit.NSImageNameShareTemplate, str)

    @min_os_level("10.12")
    def testConstants10_12(self):
        self.assertIsInstance(AppKit.NSImageHintUserInterfaceLayoutDirection, str)
        self.assertIsInstance(AppKit.NSImageNameGoForwardTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameGoBackTemplate, str)

        self.assertIsInstance(AppKit.NSImageNameTouchBarAddDetailTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarAddTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarAlarmTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarAudioInputMuteTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarAudioInputTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarAudioOutputMuteTemplate, str)
        self.assertIsInstance(
            AppKit.NSImageNameTouchBarAudioOutputVolumeHighTemplate, str
        )
        self.assertIsInstance(
            AppKit.NSImageNameTouchBarAudioOutputVolumeLowTemplate, str
        )
        self.assertIsInstance(
            AppKit.NSImageNameTouchBarAudioOutputVolumeMediumTemplate, str
        )
        self.assertIsInstance(
            AppKit.NSImageNameTouchBarAudioOutputVolumeOffTemplate, str
        )
        self.assertIsInstance(AppKit.NSImageNameTouchBarBookmarksTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarColorPickerFill, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarColorPickerFont, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarColorPickerStroke, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarCommunicationAudioTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarCommunicationVideoTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarComposeTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarDeleteTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarDownloadTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarEnterFullScreenTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarExitFullScreenTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarFastForwardTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarFolderCopyToTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarFolderMoveToTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarFolderTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarGetInfoTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarGoBackTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarGoDownTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarGoForwardTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarGoUpTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarHistoryTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarIconViewTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarListViewTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarMailTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarNewFolderTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarNewMessageTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarOpenInBrowserTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarPauseTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarPlayheadTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarPlayPauseTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarPlayTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarQuickLookTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarRecordStartTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarRecordStopTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarRefreshTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarRewindTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarRotateLeftTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarRotateRightTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarSearchTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarShareTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarSidebarTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarSkipAhead15SecondsTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarSkipAhead30SecondsTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarSkipAheadTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarSkipBack15SecondsTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarSkipBack30SecondsTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarSkipBackTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarSkipToEndTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarSkipToStartTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarSlideshowTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarTagIconTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarTextBoldTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarTextBoxTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarTextCenterAlignTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarTextItalicTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarTextJustifiedAlignTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarTextLeftAlignTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarTextListTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarTextRightAlignTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarTextStrikethroughTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarTextUnderlineTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarUserAddTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarUserGroupTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarUserTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarVolumeDownTemplate, str)
        self.assertIsInstance(AppKit.NSImageNameTouchBarVolumeUpTemplate, str)

    @min_os_level("10.13")
    def testConstants10_13(self):
        self.assertIsInstance(AppKit.NSImageNameTouchBarRemoveTemplate, str)



import Foundation
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSLengthFormatter(TestCase):
    @min_os_level("10.10")
    def testConstants10_10(self):
        self.assertEqual(Foundation.NSLengthFormatterUnitMillimeter, 8)
        self.assertEqual(Foundation.NSLengthFormatterUnitCentimeter, 9)
        self.assertEqual(Foundation.NSLengthFormatterUnitMeter, 11)
        self.assertEqual(Foundation.NSLengthFormatterUnitKilometer, 14)
        self.assertEqual(Foundation.NSLengthFormatterUnitInch, (5 << 8) + 1)
        self.assertEqual(Foundation.NSLengthFormatterUnitFoot, (5 << 8) + 2)
        self.assertEqual(Foundation.NSLengthFormatterUnitYard, (5 << 8) + 3)
        self.assertEqual(Foundation.NSLengthFormatterUnitMile, (5 << 8) + 4)

    @min_os_level("10.10")
    def testMethods10_10(self):
        self.assertResultIsBOOL(Foundation.NSLengthFormatter.isForPersonHeightUse)
        self.assertArgIsBOOL(Foundation.NSLengthFormatter.setForPersonHeightUse_, 0)

        self.assertResultIsBOOL(
            Foundation.NSLengthFormatter.getObjectValue_forString_errorDescription_
        )
        self.assertArgIsOut(
            Foundation.NSLengthFormatter.getObjectValue_forString_errorDescription_, 0
        )
        self.assertArgIsOut(
            Foundation.NSLengthFormatter.getObjectValue_forString_errorDescription_, 2
        )



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSStatusBarButton(TestCase):
    @min_os_level("10.10")
    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSStatusBarButton.appearsDisabled)
        self.assertArgIsBOOL(AppKit.NSStatusBarButton.setAppearsDisabled_, 0)



import Foundation
from PyObjCTools.TestSupport import TestCase, min_os_level

NSUserScriptTaskCompletionHandler = b"v@"
NSUserUnixTaskCompletionHandler = b"v@"
NSUserAppleScriptTaskCompletionHandler = b"v@@"
NSUserAutomatorTaskCompletionHandler = b"@@"


class TestNSUserScriptTask(TestCase):
    @min_os_level("10.8")
    def testMethods10_8(self):
        self.assertArgIsOut(Foundation.NSUserScriptTask.initWithURL_error_, 1)
        self.assertArgIsBlock(
            Foundation.NSUserScriptTask.executeWithCompletionHandler_,
            0,
            NSUserScriptTaskCompletionHandler,
        )
        self.assertArgIsBlock(
            Foundation.NSUserUnixTask.executeWithArguments_completionHandler_,
            1,
            NSUserUnixTaskCompletionHandler,
        )
        self.assertArgIsBlock(
            Foundation.NSUserAppleScriptTask.executeWithAppleEvent_completionHandler_,
            1,
            NSUserAppleScriptTaskCompletionHandler,
        )
        self.assertArgIsBlock(
            Foundation.NSUserAutomatorTask.executeWithInput_completionHandler_,
            1,
            NSUserAutomatorTaskCompletionHandler,
        )



import Foundation
from PyObjCTools.TestSupport import TestCase


class TestNSGarbageCollector(TestCase):
    def testMethods(self):
        self.assertResultIsBOOL(Foundation.NSGarbageCollector.isCollecting)
        self.assertResultIsBOOL(Foundation.NSGarbageCollector.isEnabled)



import Foundation
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSNotification(TestCase):
    def testMethods(self):
        self.assertArgIsSEL(
            Foundation.NSNotificationCenter.addObserver_selector_name_object_,
            1,
            b"v@:@",
        )

    @min_os_level("10.6")
    def testMethods10_6(self):
        self.assertArgIsBlock(
            Foundation.NSNotificationCenter.addObserverForName_object_queue_usingBlock_,
            3,
            b"v@",
        )



import AppKit
from PyObjCTools.TestSupport import TestCase


class TestNSArrayControler(TestCase):
    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSArrayController.automaticallyRearrangesObjects)

        self.assertResultIsBOOL(
            AppKit.NSArrayController.clearsFilterPredicateOnInsertion
        )
        self.assertArgIsBOOL(
            AppKit.NSArrayController.setClearsFilterPredicateOnInsertion_, 0
        )

        self.assertResultIsBOOL(AppKit.NSArrayController.avoidsEmptySelection)
        self.assertArgIsBOOL(AppKit.NSArrayController.setAvoidsEmptySelection_, 0)

        self.assertResultIsBOOL(AppKit.NSArrayController.preservesSelection)
        self.assertArgIsBOOL(AppKit.NSArrayController.setPreservesSelection_, 0)

        self.assertResultIsBOOL(AppKit.NSArrayController.selectsInsertedObjects)
        self.assertArgIsBOOL(AppKit.NSArrayController.setSelectsInsertedObjects_, 0)

        self.assertResultIsBOOL(AppKit.NSArrayController.alwaysUsesMultipleValuesMarker)
        self.assertArgIsBOOL(
            AppKit.NSArrayController.setAlwaysUsesMultipleValuesMarker_, 0
        )

        self.assertResultIsBOOL(AppKit.NSArrayController.setSelectionIndexes_)
        self.assertResultIsBOOL(AppKit.NSArrayController.setSelectionIndex_)
        self.assertResultIsBOOL(AppKit.NSArrayController.addSelectionIndexes_)
        self.assertResultIsBOOL(AppKit.NSArrayController.removeSelectionIndexes_)
        self.assertResultIsBOOL(AppKit.NSArrayController.setSelectedObjects_)
        self.assertResultIsBOOL(AppKit.NSArrayController.addSelectedObjects_)
        self.assertResultIsBOOL(AppKit.NSArrayController.removeSelectedObjects_)
        self.assertResultIsBOOL(AppKit.NSArrayController.canInsert)
        self.assertResultIsBOOL(AppKit.NSArrayController.canSelectNext)
        self.assertResultIsBOOL(AppKit.NSArrayController.canSelectPrevious)



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level
import objc


class TestNSDocument(TestCase):
    def testConstants(self):
        self.assertEqual(AppKit.NSChangeDone, 0)
        self.assertEqual(AppKit.NSChangeUndone, 1)
        self.assertEqual(AppKit.NSChangeCleared, 2)
        self.assertEqual(AppKit.NSChangeRedone, 5)
        self.assertEqual(AppKit.NSChangeReadOtherContents, 3)
        self.assertEqual(AppKit.NSChangeAutosaved, 4)

        self.assertEqual(AppKit.NSSaveOperation, 0)
        self.assertEqual(AppKit.NSSaveAsOperation, 1)
        self.assertEqual(AppKit.NSSaveToOperation, 2)
        self.assertEqual(AppKit.NSAutosaveOperation, 3)

    @min_os_level("10.7")
    def testConstants10_7(self):
        self.assertEqual(AppKit.NSChangeDiscardable, 256)
        self.assertEqual(AppKit.NSAutosaveInPlaceOperation, 4)
        self.assertEqual(AppKit.NSAutosaveElsewhereOperation, 3)

    @min_os_level("10.8")
    def testConstants10_8(self):
        self.assertEqual(AppKit.NSAutosaveAsOperation, 5)

    def testMethods(self):
        self.assertArgIsOut(AppKit.NSDocument.initWithType_error_, 1)
        self.assertArgIsOut(AppKit.NSDocument.initWithContentsOfURL_ofType_error_, 2)
        self.assertArgIsOut(
            AppKit.NSDocument.initForURL_withContentsOfURL_ofType_error_, 3
        )
        self.assertResultIsBOOL(AppKit.NSDocument.revertToContentsOfURL_ofType_error_)
        self.assertArgIsOut(AppKit.NSDocument.revertToContentsOfURL_ofType_error_, 2)
        self.assertResultIsBOOL(AppKit.NSDocument.readFromURL_ofType_error_)
        self.assertArgIsOut(AppKit.NSDocument.readFromURL_ofType_error_, 2)
        self.assertResultIsBOOL(AppKit.NSDocument.readFromFileWrapper_ofType_error_)
        self.assertArgIsOut(AppKit.NSDocument.readFromFileWrapper_ofType_error_, 2)
        self.assertResultIsBOOL(AppKit.NSDocument.readFromData_ofType_error_)
        self.assertArgIsOut(AppKit.NSDocument.readFromData_ofType_error_, 2)
        self.assertResultIsBOOL(AppKit.NSDocument.writeToURL_ofType_error_)
        self.assertArgIsOut(AppKit.NSDocument.writeToURL_ofType_error_, 2)
        self.assertArgIsOut(AppKit.NSDocument.fileWrapperOfType_error_, 1)
        self.assertArgIsOut(AppKit.NSDocument.dataOfType_error_, 1)
        self.assertResultIsBOOL(
            AppKit.NSDocument.writeSafelyToURL_ofType_forSaveOperation_error_
        )
        self.assertArgIsOut(
            AppKit.NSDocument.writeSafelyToURL_ofType_forSaveOperation_error_, 3
        )
        self.assertResultIsBOOL(
            AppKit.NSDocument.writeToURL_ofType_forSaveOperation_originalContentsURL_error_
        )
        self.assertArgIsOut(
            AppKit.NSDocument.writeToURL_ofType_forSaveOperation_originalContentsURL_error_,
            4,
        )
        self.assertArgIsOut(
            AppKit.NSDocument.fileAttributesToWriteToURL_ofType_forSaveOperation_originalContentsURL_error_,  # noqa: B950
            4,
        )
        self.assertResultIsBOOL(AppKit.NSDocument.keepBackupFile)
        self.assertArgIsSEL(
            AppKit.NSDocument.saveDocumentWithDelegate_didSaveSelector_contextInfo_,
            1,
            b"v@:@" + objc._C_NSBOOL + b"^v",
        )
        self.assertArgHasType(
            AppKit.NSDocument.saveDocumentWithDelegate_didSaveSelector_contextInfo_,
            2,
            b"^v",
        )
        self.assertArgIsSEL(
            AppKit.NSDocument.runModalSavePanelForSaveOperation_delegate_didSaveSelector_contextInfo_,  # noqa: B950
            2,
            b"v@:@" + objc._C_NSBOOL + b"^v",
        )
        self.assertArgHasType(
            AppKit.NSDocument.runModalSavePanelForSaveOperation_delegate_didSaveSelector_contextInfo_,  # noqa: B950
            3,
            b"^v",
        )
        self.assertResultIsBOOL(AppKit.NSDocument.shouldRunSavePanelWithAccessoryView)
        self.assertResultIsBOOL(AppKit.NSDocument.prepareSavePanel_)
        self.assertResultIsBOOL(
            AppKit.NSDocument.fileNameExtensionWasHiddenInLastRunSavePanel
        )
        self.assertArgIsSEL(
            AppKit.NSDocument.saveToURL_ofType_forSaveOperation_delegate_didSaveSelector_contextInfo_,  # noqa: B950
            4,
            b"v@:@" + objc._C_NSBOOL + b"^v",
        )
        self.assertArgHasType(
            AppKit.NSDocument.saveToURL_ofType_forSaveOperation_delegate_didSaveSelector_contextInfo_,  # noqa: B950
            5,
            b"^v",
        )
        self.assertResultIsBOOL(
            AppKit.NSDocument.saveToURL_ofType_forSaveOperation_error_
        )
        self.assertArgIsOut(
            AppKit.NSDocument.saveToURL_ofType_forSaveOperation_error_, 3
        )
        self.assertResultIsBOOL(AppKit.NSDocument.hasUnautosavedChanges)
        self.assertArgIsSEL(
            AppKit.NSDocument.autosaveDocumentWithDelegate_didAutosaveSelector_contextInfo_,
            1,
            b"v@:@" + objc._C_NSBOOL + b"^v",
        )
        self.assertArgHasType(
            AppKit.NSDocument.autosaveDocumentWithDelegate_didAutosaveSelector_contextInfo_,
            2,
            b"^v",
        )
        self.assertArgIsSEL(
            AppKit.NSDocument.canCloseDocumentWithDelegate_shouldCloseSelector_contextInfo_,
            1,
            b"v@:@" + objc._C_NSBOOL + b"^v",
        )
        self.assertArgHasType(
            AppKit.NSDocument.canCloseDocumentWithDelegate_shouldCloseSelector_contextInfo_,
            2,
            b"^v",
        )
        self.assertArgIsSEL(
            AppKit.NSDocument.runModalPageLayoutWithPrintInfo_delegate_didRunSelector_contextInfo_,  # noqa: B950
            2,
            b"v@:@" + objc._C_NSBOOL + b"^v",
        )
        self.assertArgHasType(
            AppKit.NSDocument.runModalPageLayoutWithPrintInfo_delegate_didRunSelector_contextInfo_,  # noqa: B950
            3,
            b"^v",
        )
        self.assertResultIsBOOL(AppKit.NSDocument.preparePageLayout_)
        self.assertResultIsBOOL(AppKit.NSDocument.shouldChangePrintInfo_)
        self.assertArgIsSEL(
            AppKit.NSDocument.printDocumentWithSettings_showPrintPanel_delegate_didPrintSelector_contextInfo_,  # noqa: B950
            3,
            b"v@:@" + objc._C_NSBOOL + b"^v",
        )
        self.assertArgHasType(
            AppKit.NSDocument.printDocumentWithSettings_showPrintPanel_delegate_didPrintSelector_contextInfo_,  # noqa: B950
            4,
            b"^v",
        )
        self.assertArgIsOut(AppKit.NSDocument.printOperationWithSettings_error_, 1)
        self.assertArgIsSEL(
            AppKit.NSDocument.runModalPrintOperation_delegate_didRunSelector_contextInfo_,
            2,
            b"v@:@" + objc._C_NSBOOL + b"^v",
        )
        self.assertArgHasType(
            AppKit.NSDocument.runModalPrintOperation_delegate_didRunSelector_contextInfo_,
            3,
            b"^v",
        )
        self.assertResultIsBOOL(AppKit.NSDocument.isDocumentEdited)
        self.assertResultIsBOOL(AppKit.NSDocument.hasUndoManager)
        self.assertArgIsBOOL(AppKit.NSDocument.setHasUndoManager_, 0)
        self.assertResultIsBOOL(AppKit.NSDocument.presentError_)
        self.assertArgIsSEL(
            AppKit.NSDocument.presentError_modalForWindow_delegate_didPresentSelector_contextInfo_,  # noqa: B950
            3,
            b"v@:" + objc._C_NSBOOL + b"^v",
        )
        self.assertArgHasType(
            AppKit.NSDocument.presentError_modalForWindow_delegate_didPresentSelector_contextInfo_,  # noqa: B950
            4,
            b"^v",
        )
        self.assertArgIsSEL(
            AppKit.NSDocument.shouldCloseWindowController_delegate_shouldCloseSelector_contextInfo_,  # noqa: B950
            2,
            b"v@:@" + objc._C_NSBOOL + b"^v",
        )
        self.assertArgHasType(
            AppKit.NSDocument.shouldCloseWindowController_delegate_shouldCloseSelector_contextInfo_,  # noqa: B950
            3,
            b"^v",
        )
        self.assertResultIsBOOL(AppKit.NSDocument.isNativeType_)
        self.assertResultIsBOOL(AppKit.NSDocument.validateUserInterfaceItem_)
        self.assertResultIsBOOL(AppKit.NSDocument.loadDataRepresentation_ofType_)
        self.assertArgIsBOOL(AppKit.NSDocument.printShowingPrintPanel_, 0)
        self.assertResultIsBOOL(AppKit.NSDocument.readFromFile_ofType_)
        self.assertResultIsBOOL(AppKit.NSDocument.readFromURL_ofType_)
        self.assertResultIsBOOL(AppKit.NSDocument.revertToSavedFromFile_ofType_)
        self.assertResultIsBOOL(AppKit.NSDocument.revertToSavedFromURL_ofType_)
        self.assertResultIsBOOL(AppKit.NSDocument.writeToFile_ofType_)
        self.assertResultIsBOOL(
            AppKit.NSDocument.writeToFile_ofType_originalFile_saveOperation_
        )
        self.assertResultIsBOOL(AppKit.NSDocument.writeToURL_ofType_)
        self.assertResultIsBOOL(
            AppKit.NSDocument.writeWithBackupToFile_ofType_saveOperation_
        )
        self.assertArgIsSEL(
            AppKit.NSDocument.saveToFile_saveOperation_delegate_didSaveSelector_contextInfo_,
            3,
            b"v@:@" + objc._C_NSBOOL + b"^v",
        )
        self.assertArgHasType(
            AppKit.NSDocument.saveToFile_saveOperation_delegate_didSaveSelector_contextInfo_,
            4,
            b"^v",
        )

    @min_os_level("10.6")
    def testMethods10_6(self):
        self.assertResultIsBOOL(AppKit.NSDocument.canConcurrentlyReadDocumentsOfType_)

    @min_os_level("10.7")
    def testMethods10_7(self):
        self.assertArgIsBOOL(
            AppKit.NSDocument.performActivityWithSynchronousWaiting_usingBlock_, 0
        )
        self.assertArgIsBlock(
            AppKit.NSDocument.performActivityWithSynchronousWaiting_usingBlock_, 1, b"v"
        )
        self.assertArgIsBlock(AppKit.NSDocument.continueActivityUsingBlock_, 0, b"v")
        self.assertArgIsBlock(
            AppKit.NSDocument.continueAsynchronousWorkOnMainThreadUsingBlock_, 0, b"v"
        )
        self.assertArgIsBlock(
            AppKit.NSDocument.performSynchronousFileAccessUsingBlock_, 0, b"v"
        )

        self.assertArgIsBlock(
            AppKit.NSDocument.performAsynchronousFileAccessUsingBlock_, 0, b"v@?"
        )  # FIXME: block has a block argument

        self.assertResultIsBOOL(AppKit.NSDocument.isEntireFileLoaded)
        self.assertResultIsBOOL(AppKit.NSDocument.autosavingIsImplicitlyCancellable)

        self.assertArgIsBlock(
            AppKit.NSDocument.saveToURL_ofType_forSaveOperation_completionHandler_,
            3,
            b"v@",
        )
        self.assertResultIsBOOL(
            AppKit.NSDocument.canAsynchronouslyWriteToURL_ofType_forSaveOperation_
        )
        self.assertResultIsBOOL(AppKit.NSDocument.checkAutosavingSafetyAndReturnError_)
        self.assertArgIsOut(AppKit.NSDocument.checkAutosavingSafetyAndReturnError_, 0)
        self.assertArgIsBOOL(
            AppKit.NSDocument.autosaveWithImplicitCancellability_completionHandler_, 0
        )
        self.assertArgIsBlock(
            AppKit.NSDocument.autosaveWithImplicitCancellability_completionHandler_,
            1,
            b"v@",
        )

        self.assertResultIsBOOL(AppKit.NSDocument.autosavesInPlace)
        self.assertResultIsBOOL(AppKit.NSDocument.preservesVersions)

        self.assertArgIsSEL(
            AppKit.NSDocument.duplicateDocumentWithDelegate_didDuplicateSelector_contextInfo_,
            1,
            b"v@" + objc._C_NSBOOL + b"^v",
        )
        self.assertArgIsOut(AppKit.NSDocument.duplicateAndReturnError_, 0)
        self.assertResultIsBOOL(AppKit.NSDocument.isInViewingMode)

        self.assertArgIsBlock(
            AppKit.NSDocument.saveToURL_ofType_forSaveOperation_completionHandler_,
            3,
            b"v@",
        )

    @min_os_level("10.8")
    def testMethods10_8(self):
        self.assertArgIsBOOL(AppKit.NSDocument.setDraft_, 0)
        self.assertResultIsBOOL(AppKit.NSDocument.isDraft)
        self.assertResultIsBOOL(AppKit.NSDocument.autosavesDrafts)

        self.assertArgIsBlock(
            AppKit.NSDocument.moveDocumentWithCompletionHandler_,
            0,
            b"v" + objc._C_NSBOOL,
        )
        self.assertArgIsBlock(
            AppKit.NSDocument.moveToURL_completionHandler_, 1, b"v" + objc._C_NSBOOL
        )
        self.assertArgIsBlock(
            AppKit.NSDocument.lockDocumentWithCompletionHandler_,
            0,
            b"v" + objc._C_NSBOOL,
        )
        self.assertArgIsBlock(AppKit.NSDocument.lockWithCompletionHandler_, 0, b"v@")
        self.assertArgIsBlock(
            AppKit.NSDocument.unlockDocumentWithCompletionHandler_,
            0,
            b"v" + objc._C_NSBOOL,
        )
        self.assertArgIsBlock(AppKit.NSDocument.unlockWithCompletionHandler_, 0, b"v@")
        self.assertResultIsBOOL(AppKit.NSDocument.isLocked)
        self.assertResultIsBOOL(AppKit.NSDocument.usesUbiquitousStorage)

    @min_os_level("10.12")
    def testMethods10_12(self):
        self.assertResultIsBOOL(AppKit.NSDocument.isBrowsingVersions)

        self.assertArgIsBlock(
            AppKit.NSDocument.stopBrowsingVersionsWithCompletionHandler_, 0, b"v"
        )

    @min_os_level("10.13")
    def testMethods10_13(self):
        self.assertResultIsBOOL(AppKit.NSDocument.allowsDocumentSharing)
        self.assertArgIsBlock(
            AppKit.NSDocument.shareDocumentWithSharingService_completionHandler_,
            1,
            b"vZ",
        )



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSDate(TestCase):
    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSDate.isEqualToDate_)

    def testConstants(self):
        self.assertEqual(AppKit.NSTimeIntervalSince1970, 978_307_200.0)

    @min_os_level("10.6")
    def testConstants10_6(self):
        self.assertIsInstance(AppKit.NSSystemClockDidChangeNotification, str)



import AppKit
from PyObjCTools.TestSupport import TestCase


class TestNSWindowScripting(TestCase):
    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSWindow.hasCloseBox)
        self.assertResultIsBOOL(AppKit.NSWindow.hasTitleBar)
        self.assertResultIsBOOL(AppKit.NSWindow.isFloatingPanel)
        self.assertResultIsBOOL(AppKit.NSWindow.isMiniaturizable)
        self.assertResultIsBOOL(AppKit.NSWindow.isModalPanel)
        self.assertResultIsBOOL(AppKit.NSWindow.isResizable)
        self.assertResultIsBOOL(AppKit.NSWindow.isZoomable)
        self.assertArgIsBOOL(AppKit.NSWindow.setIsMiniaturized_, 0)
        self.assertArgIsBOOL(AppKit.NSWindow.setIsVisible_, 0)
        self.assertArgIsBOOL(AppKit.NSWindow.setIsZoomed_, 0)



#
#  PreferencesPanelController.py
#  ControlledPreferences
#
#  Converted by u.fiedler on 04.02.05.
#  with great help from Bob Ippolito - Thank you Bob!
#
#  The original version was written in Objective-C by Malcolm Crawford
#  at http://homepage.mac.com/mmalc/CocoaExamples/controllers.html

import objc
from Cocoa import (
    NSArchiver,
    NSColor,
    NSFont,
    NSFontManager,
    NSUserDefaultsController,
    NSValueTransformer,
    NSWindowController,
)
from FontNameToDisplayNameTransformer import FontNameToDisplayNameTransformer
from PyObjCTools.KeyValueCoding import getKey


class PreferencesPanelController(NSWindowController):
    @objc.IBAction
    def changeTextFont_(self, sender):
        """The user changed the current font selection, so update the default font"""

        # Get font name and size from user defaults
        defaults = NSUserDefaultsController.sharedUserDefaultsController().values()
        fontName = getKey(defaults, "FontName")
        fontSize = getKey(defaults, "FontSize")

        # Create font from name and size; initialize font panel
        font = NSFont.fontWithName_size_(fontName, fontSize)
        if font is None:
            font = NSFont.systemFontOfSize_(NSFont.systemFontSize())
        NSFontManager.sharedFontManager().setSelectedFont_isMultiple_(font, False)
        NSFontManager.sharedFontManager().orderFrontFontPanel_(self)

        # Set window as firstResponder so we get changeFont: messages
        self.window().makeFirstResponder_(self.window())

    @objc.IBAction
    def changeFont_(self, sender):
        """This is the message the font panel sends when a new font is selected"""
        # Get selected font
        fontManager = NSFontManager.sharedFontManager()
        selectedFont = fontManager.selectedFont()
        if selectedFont is None:
            selectedFont = NSFont.systemFontOfSize_(NSFont.systemFontSize())
        panelFont = fontManager.convertFont_(selectedFont)

        # Get and store details of selected font
        # Note: use fontName, not displayName.  The font name identifies the font to
        # the system, we use a value transformer to show the user the display name
        fontSize = panelFont.pointSize()

        defaults = NSUserDefaultsController.sharedUserDefaultsController().values()
        defaults.setValue_forKey_(panelFont.fontName(), "FontName")
        defaults.setValue_forKey_(fontSize, "FontSize")


# Set up initial values for defaults:
# Create dictionary with keys and values for WordOfTheDay, FontName,
# FontSize, and FavoriteColor.  Mostly straightforward, but:
#
# Store the fontName of the font as the default; the textfield displays
# the font's displayName using a value transformer.
#
# The color must be archived -- you can't store NSColors directly in NSUserDefaults.
dictionary = {}
dictionary["WordOfTheDay"] = "Today"
systemFont = NSFont.systemFontOfSize_(NSFont.systemFontSize())
dictionary["FontName"] = systemFont.fontName()
dictionary["FontSize"] = systemFont.pointSize()
archivedColor = NSArchiver.archivedDataWithRootObject_(NSColor.greenColor())
dictionary["FavoriteColor"] = archivedColor
NSUserDefaultsController.sharedUserDefaultsController().setInitialValues_(dictionary)

# Create and register font name value transformer
transformer = FontNameToDisplayNameTransformer.alloc().init()
NSValueTransformer.setValueTransformer_forName_(
    transformer, "FontNameToDisplayNameTransformer"
)



import Foundation
from PyObjCTools.TestSupport import TestCase


class TestNSRelativeDateTimeFormatter(TestCase):
    def test_constants(self):
        self.assertEqual(Foundation.NSRelativeDateTimeFormatterStyleNumeric, 0)
        self.assertEqual(Foundation.NSRelativeDateTimeFormatterStyleNamed, 1)

        self.assertEqual(Foundation.NSRelativeDateTimeFormatterUnitsStyleFull, 0)
        self.assertEqual(Foundation.NSRelativeDateTimeFormatterUnitsStyleSpellOut, 1)
        self.assertEqual(Foundation.NSRelativeDateTimeFormatterUnitsStyleShort, 2)
        self.assertEqual(Foundation.NSRelativeDateTimeFormatterUnitsStyleAbbreviated, 3)



import AppKit
from PyObjCTools.TestSupport import TestCase


class TestNSBitmapImageRep(TestCase):
    def test_getTIFFCompressionTypes(self):
        r = AppKit.NSBitmapImageRep.getTIFFCompressionTypes_count_(None, None)

        self.assertIsInstance(r, tuple)
        self.assertEqual(len(r), 2)

        lst, ln = r

        self.assertIsInstance(lst, tuple)
        self.assertIsInstance(ln, int)

        self.assertEqual(len(lst), ln)
        self.assertTrue(all(isinstance(x, int) for x in lst))



import sys

import CoreFoundation
from PyObjCTools.TestSupport import TestCase

if sys.version_info[0] == 3:

    def cmp(a, b):
        try:
            if a < b:
                return -1
            elif b < a:
                return 1
            return 0

        except TypeError:
            return cmp(type(a).__name__, type(b).__name__)


class TestCFTree(TestCase):
    def testTypes(self):
        self.assertIsCFType(CoreFoundation.CFTreeRef)

    def testCreation(self):
        context = object()
        tree = CoreFoundation.CFTreeCreate(None, context)

        self.assertIsInstance(tree, CoreFoundation.CFTreeRef)

        self.assertTrue(CoreFoundation.CFTreeGetContext(tree, None) is context)
        CoreFoundation.CFTreeSetContext(tree, 42)
        self.assertEqual(CoreFoundation.CFTreeGetContext(tree, None), 42)

    def testCreateTree(self):
        root = CoreFoundation.CFTreeCreate(None, "root")

        for child in range(10):
            CoreFoundation.CFTreeAppendChild(
                root, CoreFoundation.CFTreeCreate(None, child)
            )

        self.assertEqual(
            CoreFoundation.CFTreeGetContext(
                CoreFoundation.CFTreeGetFirstChild(root), None
            ),
            0,
        )

        def compare(a, b, context):
            return -cmp(
                CoreFoundation.CFTreeGetContext(a, None),
                CoreFoundation.CFTreeGetContext(b, None),
            )

        CoreFoundation.CFTreeSortChildren(root, compare, None)
        self.assertEqual(
            CoreFoundation.CFTreeGetContext(
                CoreFoundation.CFTreeGetFirstChild(root), None
            ),
            9,
        )

    def testTypeID(self):
        v = CoreFoundation.CFTreeGetTypeID()
        self.assertIsInstance(v, int)

    def testQuerying(self):
        root = CoreFoundation.CFTreeCreate(None, "root")

        for child in range(2):
            CoreFoundation.CFTreeAppendChild(
                root, CoreFoundation.CFTreeCreate(None, child)
            )

        p = CoreFoundation.CFTreeGetParent(root)
        self.assertIs(p, None)
        c = CoreFoundation.CFTreeGetFirstChild(root)
        self.assertIsInstance(c, CoreFoundation.CFTreeRef)
        c2 = CoreFoundation.CFTreeGetChildAtIndex(root, 0)
        self.assertIs(c, c2)
        p = CoreFoundation.CFTreeGetParent(c)
        self.assertIs(p, root)
        s = CoreFoundation.CFTreeGetNextSibling(c)
        self.assertIsInstance(s, CoreFoundation.CFTreeRef)
        p = CoreFoundation.CFTreeGetParent(s)
        self.assertIs(p, root)
        s2 = CoreFoundation.CFTreeGetChildAtIndex(root, 1)
        self.assertIs(s, s2)
        s = CoreFoundation.CFTreeGetNextSibling(s)
        self.assertIs(s, None)
        cnt = CoreFoundation.CFTreeGetChildCount(root)
        self.assertEqual(cnt, 2)

        cnt = CoreFoundation.CFTreeGetChildCount(c)
        self.assertEqual(cnt, 0)

        children = CoreFoundation.CFTreeGetChildren(root, None)
        self.assertIsInstance(children, (list, tuple))
        self.assertEqual(len(children), 2)

        r = CoreFoundation.CFTreeFindRoot(s2)
        self.assertIs(r, root)

    def testModification(self):
        root = CoreFoundation.CFTreeCreate(None, "root")

        for child in range(2):
            CoreFoundation.CFTreeAppendChild(
                root, CoreFoundation.CFTreeCreate(None, child)
            )

        def applyFunc(node, context):
            context.append(CoreFoundation.CFTreeGetContext(node, None))

        lst = []
        CoreFoundation.CFTreeApplyFunctionToChildren(root, applyFunc, lst)
        self.assertEqual(len(lst), 2)

        preChild = CoreFoundation.CFTreeCreate(None, "before")
        postChild = CoreFoundation.CFTreeCreate(None, "after")
        CoreFoundation.CFTreePrependChild(root, preChild)
        CoreFoundation.CFTreeAppendChild(root, postChild)

        self.assertEqual(CoreFoundation.CFTreeGetChildCount(root), 4)
        n = CoreFoundation.CFTreeGetChildAtIndex(root, 0)
        self.assertIs(n, preChild)
        n = CoreFoundation.CFTreeGetChildAtIndex(root, 3)
        self.assertIs(n, postChild)
        s = CoreFoundation.CFTreeCreate(None, "sibling")
        CoreFoundation.CFTreeInsertSibling(preChild, s)
        n = CoreFoundation.CFTreeGetChildAtIndex(root, 1)
        self.assertIs(n, s)
        self.assertEqual(CoreFoundation.CFTreeGetChildCount(root), 5)

        CoreFoundation.CFTreeRemove(s)
        self.assertEqual(CoreFoundation.CFTreeGetChildCount(root), 4)

        def compare(left, right, context):
            left = CoreFoundation.CFTreeGetContext(left, None)
            right = CoreFoundation.CFTreeGetContext(right, None)
            return -cmp(left, right)

        before = []
        after = []
        CoreFoundation.CFTreeApplyFunctionToChildren(root, applyFunc, before)
        CoreFoundation.CFTreeSortChildren(root, compare, None)
        CoreFoundation.CFTreeApplyFunctionToChildren(root, applyFunc, after)

        self.assertItemsEqual(before, after)

        CoreFoundation.CFTreeRemoveAllChildren(root)
        self.assertEqual(CoreFoundation.CFTreeGetChildCount(root), 0)



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level, min_sdk_level
import objc


class TestNSToolbarHelper(AppKit.NSObject):
    def toolbar_itemForItemIdentifier_willBeInsertedIntoToolbar_(self, a, b, c):
        return 1


class TestNSToolbar(TestCase):
    def testConstants(self):
        self.assertEqual(AppKit.NSToolbarDisplayModeDefault, 0)
        self.assertEqual(AppKit.NSToolbarDisplayModeIconAndLabel, 1)
        self.assertEqual(AppKit.NSToolbarDisplayModeIconOnly, 2)
        self.assertEqual(AppKit.NSToolbarDisplayModeLabelOnly, 3)

        self.assertEqual(AppKit.NSToolbarSizeModeDefault, 0)
        self.assertEqual(AppKit.NSToolbarSizeModeRegular, 1)
        self.assertEqual(AppKit.NSToolbarSizeModeSmall, 2)

        self.assertIsInstance(AppKit.NSToolbarWillAddItemNotification, str)
        self.assertIsInstance(AppKit.NSToolbarDidRemoveItemNotification, str)

    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSToolbar.isVisible)
        self.assertArgIsBOOL(AppKit.NSToolbar.setVisible_, 0)
        self.assertResultIsBOOL(AppKit.NSToolbar.customizationPaletteIsRunning)
        self.assertResultIsBOOL(AppKit.NSToolbar.showsBaselineSeparator)
        self.assertArgIsBOOL(AppKit.NSToolbar.setShowsBaselineSeparator_, 0)
        self.assertResultIsBOOL(AppKit.NSToolbar.allowsUserCustomization)
        self.assertArgIsBOOL(AppKit.NSToolbar.setAllowsUserCustomization_, 0)
        self.assertResultIsBOOL(AppKit.NSToolbar.autosavesConfiguration)
        self.assertArgIsBOOL(AppKit.NSToolbar.setAutosavesConfiguration_, 0)

    @min_os_level("10.10")
    def testMethods10_10(self):
        self.assertResultIsBOOL(AppKit.NSToolbar.allowsExtensionItems)
        self.assertArgIsBOOL(AppKit.NSToolbar.setAllowsExtensionItems_, 0)

    @min_sdk_level("10.6")
    def testProtocolObjects(self):
        objc.protocolNamed("NSToolbarDelegate")

    def testProtocols(self):
        self.assertArgIsBOOL(
            TestNSToolbarHelper.toolbar_itemForItemIdentifier_willBeInsertedIntoToolbar_,
            2,
        )
